# -*- coding: utf-8 -*-

from zope.interface import Interface


class IPilotService(Interface):

    def user_join(self, info):
        """
        """

    def user_left(self, info):
        """
        """

    def user_chat(self, info):
        """
        """

    def seat_occupied(self, info):
        """
        """

    def weapons_loaded(self, info):
        """
        """

    def was_killed(self, info):
        """
        """

    def was_shot_down(self, info):
        """
        """

    def selected_army(self, info):
        """
        """

    def went_to_menu(self, info):
        """
        """

    def was_destroyed(self, info):
        """
        """

    def in_flight(self, info):
        """
        """

    def landed(self, info):
        """
        """

    def damaged(self, info):
        """
        """

    def damaged_on_ground(self, info):
        """
        """

    def turned_wingtip_smokes(self, info):
        """
        """

    def crashed(self, info):
        """
        """

    def bailed_out(self, info):
        """
        """

    def was_captured(self, info):
        """
        """

    def was_wounded(self, info):
        """
        """

    def was_heavily_wounded(self, info):
        """
        """

    def removed(self, info):
        """
        """


class IObjectsService(Interface):

    def was_destroyed(self, info):
        """
        """


class IMissionService(Interface):


    def on_status_info(self, info):
        """
        """

    def began(self, info=None):
        """
        """

    def ended(self, info=None):
        """
        """



# -*- coding: utf-8 -*-

from twisted.internet import defer
from twisted.internet import error

from il2ds_middleware.parser import DeviceLinkParser
from il2ds_middleware.tests.base import BaseMiddlewareTestCase
from il2ds_middleware.ds_emulator.constants import LONG_OPERATION_CMD


class ConsoleClientFactoryConnectionFailTestCase(BaseMiddlewareTestCase):

    console_server_port = 20001
    device_link_server_port = 10001

    def setUp(self):

        def on_connection_fail(err):
            if isinstance(err.value, error.ConnectionRefusedError):
                self.console_client_connector = None

        d = super(ConsoleClientFactoryConnectionFailTestCase, self).setUp()
        return d.addErrback(on_connection_fail)

    def test_connection_fail(self):
        self.assertNot(self.console_client_connector)

    @property
    def console_client_host_for_client(self):
        return self.console_server_host, self.console_server_port+1

    @property
    def device_link_host_for_client(self):
        return self.device_link_server_host, self.device_link_server_port+1


class ConsoleClientFactoryTestCase(BaseMiddlewareTestCase):

    def test_connection(self):
        self.assertTrue(self.console_client_connector)

    def test_wrong_rid(self):
        self.console_client._process_responce_id("rid|0")

    def test_malformed_rid(self):
        self.console_client._process_responce_id("rid/smth")
        self.console_client._process_responce_id("rid|smth")

    def test_mission_status(self):

        srvc = self.service.getServiceNamed('missions')

        def do_test():
            d = self.console_client.mission_status()
            d.addCallback(check_not_loaded)
            d.addCallback(do_load)
            return d

        def do_load(_):
            srvc.load("net/dogfight/test.mis")
            d = self.console_client.mission_status()
            d.addCallback(check_loaded)
            d.addCallback(do_begin)
            return d

        def do_begin(_):
            srvc.begin()
            d = self.console_client.mission_status()
            d.addCallback(check_playing)
            return d

        def check_not_loaded(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 1)
            self.assertEqual(response[0], "Mission NOT loaded")

        def check_loaded(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 1)
            self.assertEqual(
                response[0], "Mission: net/dogfight/test.mis is Loaded")

        def check_playing(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 1)
            self.assertEqual(
                response[0], "Mission: net/dogfight/test.mis is Playing")

        return do_test()

    def test_server_info(self):

        def callback(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 3)
            self.assertEqual(response[0], "Type: Local server")
            self.assertEqual(response[1], "Name: Server")
            self.assertEqual(response[2], "Description: ")

        d = self.console_client.server_info()
        return d.addCallback(callback)

    def test_long_operation(self):
        d = self.console_client._send_request(LONG_OPERATION_CMD)
        return self.assertFailure(d, defer.TimeoutError)

    def test_manual_input(self):

        def do_test():
            results = [
                "mission",
                "Mission NOT loaded", ]
            d = defer.Deferred()
            self._set_console_expecting_receiver(results, d)
            self.service.manual_input("mission")
            return d.addCallback(wait_a_bit)

        def wait_a_bit(_):
            """Wait to receive <consoleN><0>"""
            d = defer.Deferred()
            from twisted.internet import reactor
            reactor.callLater(0.05, d.callback, None)
            return d

        return do_test()

    def test_mission_load(self):

        def callback(response):
            self.assertIsInstance(response, list)
            obligatory_responses = [
                "Loading mission net/dogfight/test.mis...",
                "Load bridges",
                "Load static objects",
                "Mission: net/dogfight/test.mis is Loaded", ]
            for obligatory_response in obligatory_responses:
                self.assertIn(obligatory_response, response)
                self.assertEqual(response.count(obligatory_response), 1)

        d = self.console_client.mission_load("net/dogfight/test.mis")
        return d.addCallback(callback)

    def test_mission_begin(self):

        def do_test():
            d = self.console_client.mission_load("net/dogfight/test.mis")
            return d.addCallback(do_begin)

        def do_begin(_):
            d = self.console_client.mission_begin()
            return d.addCallback(callback)

        def callback(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 1)
            self.assertEqual(
                response[0], "Mission: net/dogfight/test.mis is Playing")

        return do_test()

    def test_mission_end(self):

        def do_test():
            d = self.console_client.mission_load("net/dogfight/test.mis")
            return d.addCallback(do_begin)

        def do_begin(_):
            d = self.console_client.mission_begin()
            return d.addCallback(do_end)

        def do_end(_):
            d = self.console_client.mission_end()
            return d.addCallback(callback)

        def callback(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 1)
            self.assertEqual(
                response[0], "Mission: net/dogfight/test.mis is Loaded")

        return do_test()

    def test_mission_destroy(self):

        def do_test():
            d = self.console_client.mission_load("net/dogfight/test.mis")
            return d.addCallback(do_begin)

        def do_begin(_):
            d = self.console_client.mission_begin()
            return d.addCallback(do_destroy)

        def do_destroy(_):
            d = self.console_client.mission_destroy()
            return d.addCallback(callback)

        def callback(response):
            self.assertIsInstance(response, list)
            self.assertEqual(len(response), 1)
            self.assertEqual(response[0], "Mission NOT loaded")

        return do_test()

    def test_chat(self):
        results = ["Chat: Server: \ttest message"]*3
        results.append("Command not found: chat test message")
        d = defer.Deferred()
        self._set_console_expecting_receiver(results, d)
        self.console_client.chat_all("test message")
        self.console_client.chat_user("test message", "user0")
        self.console_client.chat_army("test message", "0")
        self.console_client.sendLine("chat test message")
        return d


class DeviceLinkClientProtocolBaseTestCase(BaseMiddlewareTestCase):

    def setUp(self):
        d = BaseMiddlewareTestCase.setUp(self)
        self.pilots = self.service.getServiceNamed('pilots')
        self.static = self.service.getServiceNamed('static')
        return d

    def _spawn_pilots(self):
        for i in xrange(2, 255):
            callsign = "user{0}".format(i-2)
            self.pilots.join(callsign, "192.168.1.{0}".format(i))
            self.pilots.spawn(callsign, pos={
                'x': i*100, 'y': i*200, 'z': i*300, })

    def _spawn_static(self):
        for i in xrange(1000):
            self.static.spawn("{0}_Static".format(i), pos={
                'x': i*100, 'y': i*200, 'z': i*300, })


class DeviceLinkClientProtocolTestCase(DeviceLinkClientProtocolBaseTestCase):

    def test_long_operation(self):
        d = self.dl_client._deferred_request((LONG_OPERATION_CMD, None))
        return self.assertFailure(d, defer.TimeoutError)

    def test_pilot_count(self):

        def on_count(response):
            self.assertEqual(response, '0')
            self._spawn_pilots()
            return self.dl_client.refresh_radar().addCallback(
                lambda _: self.dl_client.pilot_count().addCallback(
                    on_recount))

        def on_recount(response):
            self.assertEqual(response, '253')

        return self.dl_client.pilot_count().addCallback(on_count)

    def test_pilot_pos(self):

        def on_pos(response):
            self.assertEqual(response, '0:user0_0;100;200;300')

        self.pilots.join("user0", "192.168.1.{0}")
        self.pilots.spawn("user0", pos={
            'x': 100, 'y': 200, 'z': 300, })
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.pilot_pos(0).addCallback(
                on_pos))

    def test_all_pilots_pos(self):

        def on_pos_list(responses):
            self.assertIsInstance(responses, list)
            self.assertEqual(len(responses), 253)
            checked = []
            for s in responses:
                idx = int(s[s.index('user')+4:s.index(';')].split('_')[0])
                self.assertNotIn(idx, checked)
                checked.append(idx)

        self._spawn_pilots()
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.all_pilots_pos().addCallback(
                on_pos_list))

    def test_static_count(self):

        def on_count(response):
            self.assertEqual(response, '0')
            self._spawn_static()
            return self.dl_client.refresh_radar().addCallback(
                lambda _: self.dl_client.static_count().addCallback(
                    on_recount))

        def on_recount(response):
            self.assertEqual(response, '1000')

        return self.dl_client.static_count().addCallback(on_count)

    def test_static_pos(self):

        def on_pos(response):
            self.assertEqual(response, '0:0_Static;100;200;300')

        self.static.spawn("0_Static", pos={
            'x': 100, 'y': 200, 'z': 300, })
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.static_pos(0).addCallback(
                on_pos))

    def test_all_static_pos(self):

        def on_pos_list(responses):
            self.assertIsInstance(responses, list)
            self.assertEqual(len(responses), 1000)
            checked = []
            for s in responses:
                idx = int(s[s.index(':')+1:s.index('_')])
                self.assertNotIn(idx, checked)
                checked.append(idx)

        self._spawn_static()
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.all_static_pos().addCallback(
                on_pos_list))

    def test_all_pos_with_no_count(self):

        def get_count():
            d = defer.Deferred()
            d.callback(0)
            return d

        def callback(result):
            self.assertIsInstance(result, list)
            self.assertEqual(result, [])

        return self.dl_client._all_pos(get_count, 'foo').addCallback(callback)


class PasredDeviceLinkClientProtocolTestCase(
    DeviceLinkClientProtocolBaseTestCase):

    dl_client_parser_class = DeviceLinkParser

    def test_pilot_count(self):

        def on_count(response):
            self.assertEqual(response, 0)
            self._spawn_pilots()
            return self.dl_client.refresh_radar().addCallback(
                lambda _: self.dl_client.pilot_count().addCallback(
                    on_recount))

        def on_recount(response):
            self.assertEqual(response, 253)

        return self.dl_client.pilot_count().addCallback(on_count)

    def test_pilot_pos(self):

        def on_pos(response):
            self.assertIsInstance(response, dict)
            self.assertEqual(response.get('idx'), 0)
            self.assertEqual(response.get('callsign'), "user0")
            self.assertIsInstance(response.get('pos'), dict)
            self.assertEqual(response['pos'].get('x'), 100)
            self.assertEqual(response['pos'].get('y'), 200)
            self.assertEqual(response['pos'].get('z'), 300)

        self.pilots.join("user0", "192.168.1.{0}")
        self.pilots.spawn("user0", pos={
            'x': 100, 'y': 200, 'z': 300, })
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.pilot_pos(0).addCallback(
                on_pos))

    def test_all_pilots_pos(self):

        def on_pos_list(responses):
            self.assertIsInstance(responses, list)
            self.assertEqual(len(responses), 253)
            checked = []
            for response in responses:
                self.assertIsInstance(response, dict)
                idx = response['idx']
                self.assertNotIn(idx, checked)
                checked.append(idx)

        self._spawn_pilots()
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.all_pilots_pos().addCallback(
                on_pos_list))

    def test_static_count(self):

        def on_count(response):
            self.assertEqual(response, 0)
            self._spawn_static()
            return self.dl_client.refresh_radar().addCallback(
                lambda _: self.dl_client.static_count().addCallback(
                    on_recount))

        def on_recount(response):
            self.assertEqual(response, 1000)

        return self.dl_client.static_count().addCallback(on_count)

    def test_static_pos(self):

        def on_pos(response):
            self.assertIsInstance(response, dict)
            self.assertEqual(response.get('idx'), 0)
            self.assertEqual(response.get('name'), "0_Static")
            self.assertIsInstance(response.get('pos'), dict)
            self.assertEqual(response['pos'].get('x'), 100)
            self.assertEqual(response['pos'].get('y'), 200)
            self.assertEqual(response['pos'].get('z'), 300)

        self.static.spawn("0_Static", pos={
            'x': 100, 'y': 200, 'z': 300, })
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.static_pos(0).addCallback(
                on_pos))

    def test_all_static_pos(self):

        def on_pos_list(responses):
            self.assertIsInstance(responses, list)
            self.assertEqual(len(responses), 1000)
            checked = []
            for response in responses:
                self.assertIsInstance(response, dict)
                idx = response['idx']
                self.assertNotIn(idx, checked)
                checked.append(idx)

        self._spawn_static()
        return self.dl_client.refresh_radar().addCallback(
            lambda _: self.dl_client.all_static_pos().addCallback(
                on_pos_list))



# -*- coding: utf-8 -*-

from zope.interface import Interface


class ILineParser(Interface):

    def parse_line(self, line):
        """
        Parse line.

        :param line: line to parse.
        :type line: str.
        """


class IConsoleParser(ILineParser):

    def server_info(self, lines):
        """
        """

    def mission_status(self, lines):
        """
        """

    def user_joined(self, line):
        """
        """

    def user_left(self, line):
        """
        """

    def user_chat(self, line):
        """
        """


class IEventLogParser(ILineParser):

    def seat_occupied(self, data):
        """
        """

    def weapons_loaded(self, data):
        """
        """

    def was_killed(self, data):
        """
        """

    def was_shot_down(self, data):
        """
        """

    def selected_army(self, data):
        """
        """

    def went_to_menu(self, data):
        """
        """

    def was_destroyed(self, data):
        """
        """

    def in_flight(self, data):
        """
        """

    def landed(self, data):
        """
        """

    def damaged(self, data):
        """
        """

    def damaged_on_ground(self, data):
        """
        """

    def turned_wingtip_smokes(self, data):
        """
        """

    def crashed(self, data):
        """
        """

    def bailed_out(self, data):
        """
        """

    def was_captured(self, data):
        """
        """

    def was_wounded(self, data):
        """
        """

    def was_heavily_wounded(self, data):
        """
        """

    def removed(self, data):
        """
        """


class IDeviceLinkParser(Interface):

    def pilot_count(self, data):
        """
        """

    def pilot_pos(self, data):
        """
        """

    def all_pilots_pos(self, datas):
        """
        """

    def static_count(self, data):
        """
        """

    def static_pos(self, data):
        """
        """

    def all_static_pos(self, datas):
        """
        """



# -*- coding: utf-8 -*-

import optparse

from il2ds_middleware.parser import DeviceLinkParser
from il2ds_middleware.protocol import DeviceLinkClient


def parse_args():
    usage = """usage: %prog [hostname]:port"""
    parser = optparse.OptionParser(usage)

    help = "The host to connect to. Default is localhost."
    parser.add_option('--host', help=help, default='localhost')

    help = "The port to connect to. Default is 10000."
    parser.add_option('--port', type='int', default=10000, help=help)

    options, args = parser.parse_args()
    return (options.host, options.port)


def main():

    def on_pos(response):
        print "Got %s coordinates" % len(response)
        for data in response:
            print "{0}: x={1}; y={2}; z={3}".format(
                data['callsign'],
                data['pos']['x'], data['pos']['y'], data['pos']['z'], )

    def on_err(err):
        print "Getting pilots positions failed: %s." % err.value

    def do_stop(_):
        from twisted.internet import reactor
        reactor.stop()

    def on_start(_):
        client.refresh_radar()
        return client.all_pilots_pos().addCallbacks(
            on_pos, on_err).addBoth(do_stop)

    address = parse_args()
    print "Working with Device Link on %s:%s." % address

    client = DeviceLinkClient(address, DeviceLinkParser())
    client.on_start.addCallback(on_start)

    from twisted.internet import reactor
    reactor.listenUDP(0, client)
    reactor.run()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from twisted.trial.unittest import TestCase

from il2ds_middleware.constants import MISSION_STATUS, PILOT_LEAVE_REASON
from il2ds_middleware.parser import (ConsoleParser, EventLogParser,
    EventLogPassthroughParser, DeviceLinkParser, )
from il2ds_middleware.tests.service import (PilotService, ObjectsService,
    MissionService, )


class ConsoleParserTestCase(TestCase):

    def setUp(self):
        self.pilot_srvc = PilotService()
        self.mission_srvc = MissionService()
        self.parser = ConsoleParser((self.pilot_srvc, self.mission_srvc))
        self.pilot_srvc.startService()

    def tearDown(self):
        return self.pilot_srvc.stopService()

    def test_server_info(self):
        datas = [
            "Type: Local server",
            "Name: Test server",
            "Description: Horus test server", ]
        result = self.parser.server_info(datas)
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('type'), "Local server")
        self.assertEqual(result.get('name'), "Test server")
        self.assertEqual(result.get('description'), "Horus test server")

    def test_mission_status(self):
        status, mission = self.parser.mission_status([
            "Mission NOT loaded", ])
        self.assertEqual(status, MISSION_STATUS.NOT_LOADED)
        self.assertEqual(mission, None)

        status, mission = self.parser.mission_status([
            "Mission: net/dogfight/test.mis is Loaded", ])
        self.assertEqual(status, MISSION_STATUS.LOADED)
        self.assertEqual(mission, "net/dogfight/test.mis")

        status, mission = self.parser.mission_status([
            "Mission: net/dogfight/test.mis is Playing", ])
        self.assertEqual(status, MISSION_STATUS.PLAYING)
        self.assertEqual(mission, "net/dogfight/test.mis")

        datas = ["some fake data", ]
        result = self.parser.mission_status(datas)
        self.assertIsInstance(result, list)
        self.assertEqual(len(datas), len(result))
        self.assertEqual(datas[0], result[0])

    def test_mission_load(self):
        datas = [
            "Loading mission net/dogfight/test.mis...",
            "Load bridges",
            "Load static objects",
            "##### House without collision (3do/Tree/Tree2.sim)",
            "##### House without collision "
                "(3do/Buildings/Port/Floor/live.sim)",
            "##### House without collision "
                "(3do/Buildings/Port/BaseSegment/live.sim)",
            "Mission: net/dogfight/test.mis is Loaded", ]
        status, mission = self.parser.mission_status(datas)
        self.assertEqual(status, MISSION_STATUS.LOADED)
        self.assertEqual(mission, "net/dogfight/test.mis")

    def test_mission_begin(self):
        status, mission = self.parser.mission_status([
            "Mission: net/dogfight/test.mis is Playing", ])
        self.assertEqual(status, MISSION_STATUS.PLAYING)
        self.assertEqual(mission, "net/dogfight/test.mis")

    def test_mission_end(self):
        self.parser.mission_status([
            "Mission: net/dogfight/test.mis is Playing", ])
        status, mission = self.parser.mission_status([
            "Mission: net/dogfight/test.mis is Loaded", ])
        self.assertEqual(status, MISSION_STATUS.LOADED)
        self.assertEqual(mission, "net/dogfight/test.mis")

    def test_mission_destroy(self):
        status, mission = self.parser.mission_status([
            "Mission NOT loaded", ])
        self.assertEqual(status, MISSION_STATUS.NOT_LOADED)
        self.assertEqual(mission, None)

    def test_user_joined(self):
        self.parser.parse_line(
            "socket channel '0' start creating: ip 192.168.1.2:21000")
        self.parser.parse_line("Chat: --- user0 joins the game.")
        self.parser.parse_line(
            "socket channel '0', ip 192.168.1.2:21000, user0, "
            "is complete created.")
        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        info = self.pilot_srvc.buffer[0]

        self.assertIsInstance(info, dict)
        self.assertEqual(info.get('channel'), 0)
        self.assertEqual(info.get('ip'), "192.168.1.2")
        self.assertEqual(info.get('callsign'), "user0")

    def test_user_left(self):
        self.parser.parse_line(
            "socketConnection with 192.168.1.2:21000 on channel 0 lost.  "
            "Reason: ")
        self.parser.parse_line("Chat: --- user0 has left the game.")
        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        info = self.pilot_srvc.buffer[0]

        self.assertIsInstance(info, dict)
        self.assertEqual(info.get('channel'), 0)
        self.assertEqual(info.get('ip'), "192.168.1.2")
        self.assertEqual(info.get('callsign'), "user0")
        self.assertEqual(info.get('reason'), PILOT_LEAVE_REASON.DISCONNECTED)

        self.parser.parse_line(
            "socketConnection with 192.168.1.3:21000 on channel 1 lost.  "
            "Reason: You have been kicked from the server.")
        self.parser.parse_line("Chat: --- user1 has left the game.")
        self.assertEqual(len(self.pilot_srvc.buffer), 2)
        info = self.pilot_srvc.buffer[1]

        self.assertIsInstance(info, dict)
        self.assertEqual(info.get('channel'), 1)
        self.assertEqual(info.get('ip'), "192.168.1.3")
        self.assertEqual(info.get('callsign'), "user1")
        self.assertEqual(info.get('reason'), PILOT_LEAVE_REASON.KICKED)

    def test_user_chat(self):
        self.parser.parse_line("Chat: user0: \\ttest_message")
        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        info = self.pilot_srvc.buffer[0]

        self.assertIsInstance(info, tuple)
        callsign, msg = info
        self.assertEqual(callsign, "user0")
        self.assertEqual(msg, "test_message")

    def test_parse_line_mission_status(self):
        self.parser.parse_line("mission test.mis is Playing")
        self.assertEqual(len(self.mission_srvc.buffer), 1)

        info = self.mission_srvc.buffer[0]
        self.assertIsInstance(info, tuple)
        status, mission = info
        self.assertEqual(status, MISSION_STATUS.PLAYING)
        self.assertEqual(mission, "test.mis")


class EventLogPassthroughParserTestCase(TestCase):

    def setUp(self):
        self.parser = EventLogPassthroughParser()

    def test_passthrough(self):
        methods = [
            self.parser.parse_line, self.parser.seat_occupied,
            self.parser.weapons_loaded, self.parser.was_killed,
            self.parser.was_shot_down, self.parser.selected_army,
            self.parser.went_to_menu, self.parser.was_destroyed,
            self.parser.in_flight, self.parser.landed,
            self.parser.damaged, self.parser.damaged_on_ground,
            self.parser.turned_wingtip_smokes, self.parser.crashed,
            self.parser.bailed_out, self.parser.was_captured,
            self.parser.was_captured, self.parser.was_wounded,
            self.parser.was_heavily_wounded, self.parser.removed, ]
        data = "test"
        for m in methods:
            self.assertEqual(m(data), data)


class EventLogParserTestCase(TestCase):

    def setUp(self):
        self.pilot_srvc = PilotService()
        self.obj_srvc = ObjectsService()
        self.parser = EventLogParser((self.pilot_srvc, self.obj_srvc))
        self.pilot_srvc.startService()

    def tearDown(self):
        self.pilot_srvc.stopService()
        self.obj_srvc.stopService()

    def _test_pos(self, pos):
        self.assertIsInstance(pos, dict)
        self.assertEqual(pos.get('x'), 100.99)
        self.assertEqual(pos.get('y'), 200.99)

    def test_occupied_seat(self):
        data = "user0:A6M2-21(0) seat occupied by user0 at 100.99 200.99"
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('seat'), 0)
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_weapons_loaded(self):
        data = "user0:A6M2-21 loaded weapons '1xdt' fuel 100%"
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self.assertEqual(result.get('weapons'), "1xdt")
        self.assertEqual(result.get('fuel'), 100)

    def test_was_killed(self):
        data = "user0:A6M2-21(0) was killed at 100.99 200.99"
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('seat'), 0)
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def _test_was_shot_down(self, attacker):
        data = "user0:A6M2-21 shot down by {:} at 100.99 200.99".format(
            attacker)
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertIsInstance(result.get('victim'), dict)
        self.assertEqual(result['victim'].get('callsign'), "user0")
        self.assertEqual(result['victim'].get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

        self.assertIsInstance(result.get('attacker'), dict)
        return result['attacker']

    def test_was_shot_down_by_user(self):
        attacker = self._test_was_shot_down("user1:B5N2")
        self.assertEqual(attacker.get('is_user'), True)
        self.assertEqual(attacker.get('callsign'), "user1")
        self.assertEqual(attacker.get('aircraft'), "B5N2")

    def test_was_shot_down_by_ground(self):
        attacker = self._test_was_shot_down("landscape")
        self.assertEqual(attacker.get('is_user'), False)
        self.assertEqual(attacker.get('name'), "landscape")

    def test_was_shot_down_by_static(self):
        attacker = self._test_was_shot_down("0_Static")
        self.assertEqual(attacker.get('is_user'), False)
        self.assertEqual(attacker.get('name'), "0_Static")

    def test_was_shot_down_by_building(self):
        attacker = self._test_was_shot_down("0_bld")
        self.assertEqual(attacker.get('is_user'), False)
        self.assertEqual(attacker.get('name'), "0_bld")

    def test_selected_army(self):
        data = "user0 selected army Red at 100.99 200.99"
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('army'), 'Red')
        self._test_pos(result.get('pos'))

    def test_went_to_menu(self):
        data = "user0 entered refly menu"
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")

    def test_was_destroyed(self):
        data = "0_Static destroyed by landscape at 100.99 200.99"
        self.parser.parse_line(data)

        self.assertEqual(len(self.obj_srvc.buffer), 1)
        result = self.obj_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('victim'), "0_Static")
        self.assertEqual(result.get('attacker'), "landscape")
        self._test_pos(result.get('pos'))

    def test_in_flight(self):
        data = "user0:A6M2-21 in flight at 100.99 200.99"
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_landed(self):
        data = "user0:A6M2-21 landed at 100.99 200.99"
        self.parser.parse_line(data)
        data = "A6M2-21 landed at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 2)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('is_user'), True)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

        result = self.pilot_srvc.buffer[1]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('is_user'), False)
        self.assertEqual(result.get('name'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def _test_was_damaged(self, attacker):
        data = "user0:A6M2-21 damaged by {:} at 100.99 200.99".format(
            attacker)
        self.parser.parse_line(data)

        self.assertEqual(len(self.pilot_srvc.buffer), 1)
        result = self.pilot_srvc.buffer[0]

        self.assertIsInstance(result, dict)
        self.assertIsInstance(result.get('victim'), dict)
        self.assertEqual(result['victim'].get('callsign'), "user0")
        self.assertEqual(result['victim'].get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

        self.assertIsInstance(result.get('attacker'), dict)
        return result['attacker']

    def test_damaged_by_user(self):
        attacker = self._test_was_damaged("user1:B5N2")
        self.assertEqual(attacker.get('is_user'), True)
        self.assertEqual(attacker.get('callsign'), "user1")
        self.assertEqual(attacker.get('aircraft'), "B5N2")

    def test_damaged_by_landscape(self):
        attacker = self._test_was_damaged("landscape")
        self.assertEqual(attacker.get('is_user'), False)
        self.assertEqual(attacker.get('name'), "landscape")

    def test_damaged_by_static(self):
        attacker = self._test_was_damaged("0_Static")
        self.assertEqual(attacker.get('is_user'), False)
        self.assertEqual(attacker.get('name'), "0_Static")

    def test_damaged_by_building(self):
        attacker = self._test_was_damaged("0_bld")
        self.assertEqual(attacker.get('is_user'), False)
        self.assertEqual(attacker.get('name'), "0_bld")

    def test_damaged_on_ground(self):
        data = "user0:A6M2-21 damaged on the ground at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_turned_wingtip_smokes(self):
        data = "user0:A6M2-21 turned wingtip smokes on at 100.99 200.99"
        self.parser.parse_line(data)
        data = "user0:A6M2-21 turned wingtip smokes off at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 2)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self.assertEqual(result.get('state'), "on")
        self._test_pos(result.get('pos'))

        result = self.pilot_srvc.buffer[1]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self.assertEqual(result.get('state'), "off")
        self._test_pos(result.get('pos'))

    def test_crashed(self):
        data = "user0:A6M2-21 crashed at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_bailed_out(self):
        data = "user0:A6M2-21(0) bailed out at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('seat'), 0)
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_was_captured(self):
        data = "user0:A6M2-21(0) was captured at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('seat'), 0)
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_was_wounded(self):
        data = "user0:A6M2-21(0) was wounded at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('seat'), 0)
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_was_heavily_wounded(self):
        data = "user0:A6M2-21(0) was heavily wounded at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('seat'), 0)
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_removed(self):
        data = "user0:A6M2-21 removed at 100.99 200.99"
        self.parser.parse_line(data)
        self.assertEqual(len(self.pilot_srvc.buffer), 1)

        result = self.pilot_srvc.buffer[0]
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertEqual(result.get('aircraft'), "A6M2-21")
        self._test_pos(result.get('pos'))

    def test_fake_data(self):
        result = self.parser.parse_line("some fake data")
        self.assertFalse(result)


class DeviceLinkParserTestCase(TestCase):

    def setUp(self):
        self.parser = DeviceLinkParser()

    def tearDown(self):
        self.parser = None

    def test_pilot_count(self):
        result = self.parser.pilot_count('0')
        self.assertEqual(result, 0)

    def test_pilot_pos(self):
        result = self.parser.pilot_pos('0:user0_0;100;200;300')
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('idx'), 0)
        self.assertEqual(result.get('callsign'), "user0")
        self.assertIsInstance(result.get('pos'), dict)
        self.assertEqual(result['pos'].get('x'), 100)
        self.assertEqual(result['pos'].get('y'), 200)
        self.assertEqual(result['pos'].get('z'), 300)

    def test_all_pilots_pos(self):
        datas = ["{0}:user{0}_{0};{1};{2};{3}".format(
            i, i*100, i*200, i*300) for i in xrange(10)]
        results = self.parser.all_pilots_pos(datas)
        self.assertIsInstance(results, list)
        self.assertEqual(len(datas), len(results))
        for i in xrange(len(results)):
            result = results[i]
            self.assertIsInstance(result, dict)
            self.assertEqual(result.get('idx'), i)
            self.assertEqual(result.get('callsign'), "user{0}".format(i))
            self.assertIsInstance(result.get('pos'), dict)
            self.assertEqual(result['pos'].get('x'), i*100)
            self.assertEqual(result['pos'].get('y'), i*200)
            self.assertEqual(result['pos'].get('z'), i*300)

    def test_static_count(self):
        result = self.parser.static_count('0')
        self.assertEqual(result, 0)

    def test_static_pos(self):
        result = self.parser.static_pos('0:0_Static;100;200;300')
        self.assertIsInstance(result, dict)
        self.assertEqual(result.get('idx'), 0)
        self.assertEqual(result.get('name'), "0_Static")
        self.assertIsInstance(result.get('pos'), dict)
        self.assertEqual(result['pos'].get('x'), 100)
        self.assertEqual(result['pos'].get('y'), 200)
        self.assertEqual(result['pos'].get('z'), 300)

    def test_all_static_pos(self):
        datas = ["{0}:{0}_Static;{1};{2};{3}".format(
            i, i*100, i*200, i*300) for i in xrange(10)]
        results = self.parser.all_static_pos(datas)
        self.assertIsInstance(results, list)
        self.assertEqual(len(datas), len(results))
        for i in xrange(len(results)):
            result = results[i]
            self.assertIsInstance(result, dict)
            self.assertEqual(result.get('idx'), i)
            self.assertEqual(result.get('name'), "{0}_Static".format(i))
            self.assertIsInstance(result.get('pos'), dict)
            self.assertEqual(result['pos'].get('x'), i*100)
            self.assertEqual(result['pos'].get('y'), i*200)
            self.assertEqual(result['pos'].get('z'), i*300)



# -*- coding: utf-8 -*-

#-------------------------------------------------------------------------------
# Console events
#-------------------------------------------------------------------------------

"""
'socket channel '0', ip 192.168.1.2:21000, user0, is complete created.'
"""
RX_USER_JOIN = r"socket channel '(\d+)', ip (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):\d+, (.+), is complete created"
"""
'socketConnection with 192.168.1.2:21000 on channel 0 lost.  Reason: '
'socketConnection with 192.168.1.2:21000 on channel 0 lost.  Reason: You have been kicked from the server'
"""
RX_USER_LEFT = r"socketConnection with (\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):\d+ on channel (\d+) lost.  Reason: (.*)"
"""
'Chat: user0: \\ttest_message'
"""
RX_USER_CHAT = r"Chat: (.+): \\t(.*)"

#-------------------------------------------------------------------------------
# Map events
#-------------------------------------------------------------------------------

"""
' at 100.99 200.99'
"""
RX_POS = " at (\d+.\d+) (\d+.\d+)"
"""
'user0:aircraft0(0) seat occupied by user0 at 100.99 200.99'
"""
RX_SEAT_OCCUPIED = r"(.+):(.+)\((\d+)\) seat occupied by .+" + RX_POS
"""
! check AI
'user0:aircraft0 loaded weapons 'default' fuel 100%'
"""
RX_WEAPONS_LOADED = r"(.+):(.+) loaded weapons \'(.+)\' fuel (\d+)%"
"""
! check AI
'user0:aircraft0(0) was killed at 100.99 200.99'
"""
RX_KILLED = r"(.+):(.+)\((.+)\) was killed" + RX_POS
"""
! check AI
'user0:aircraft0 shot down by user1:aircraft1 at 100.99 200.99'
'user0:aircraft0 shot down by 0_bld at 100.99 200.99'
'user0:aircraft0 shot down by 0_Static at 100.99 200.99'
'user0:aircraft0 shot down by landscape at 100.99 200.99'
"""
RX_SHOT_DOWN = r"(.+):(.+) shot down by (.+)" + RX_POS
"""
'user0 selected army Red at 100.99 200.99'
"""
RX_SELECTED_ARMY = r"(.+) selected army (.+)" + RX_POS
"""
'user0 entered refly menu'
"""
RX_WENT_TO_MENU = r"(.+) entered refly menu"
"""
! check attacker
'0_Static destroyed by 1_Static at 100.99 200.99'
"""
RX_DESTROYED = r"(.+) destroyed by (.+)" + RX_POS
"""
! check AI
'user0:aircraft0 in flight at 100.99 200.99'
"""
RX_IN_FLIGHT = r"(.+):(.+) in flight" + RX_POS
"""
'user0:aircraft0 landed at 100.99 200.99'
'aircraft0 landed at 100.99 200.99'
"""
RX_LANDED = r"(.+) landed" + RX_POS
"""
! check AI
'user0:aircraft0 damaged by landscape at 100.99 200.99'
'user0:aircraft0 damaged by 0_Static at 100.99 200.99'
'user0:aircraft0 damaged by user1:aircraft1 at 100.99 200.99'
"""
RX_DAMAGED = r"(.+):(.+) damaged by (.+)" + RX_POS
"""
! check AI
'user0:aircraft0 damaged on the ground at 100.99 200.99'
"""
RX_DAMAGED_ON_GROUND = r"(.+):(.+) damaged on the ground" + RX_POS
"""
'user0:aircraft0 turned wingtip smokes on at 100.99 200.99'
'user0:aircraft0 turned wingtip smokes off at 100.99 200.99'
"""
RX_TURNED_WINGTIP_SMOKES = r"(.+):(.+) turned wingtip smokes (on|off)" + RX_POS
"""
! check AI
'user0:aircraft0 crashed at 100.99 200.99'
"""
RX_CRASHED = r"(.+):(.+) crashed" + RX_POS
"""
! check AI
'user0:aircraft0(0) bailed out at 100.99 200.99'
"""
RX_BAILED_OUT = r"(.+):(.+)\((.+)\) bailed out" + RX_POS
"""
! check AI
'user0:aircraft0(0) was captured at 100.99 200.99'
"""
RX_WAS_CAPTURED = r"(.+):(.+)\((.+)\) was captured" + RX_POS
"""
! check AI
'user0:aircraft0(0) was wounded at 100.99 200.99'
"""
RX_WAS_WOUNDED = r"(.+):(.+)\((.+)\) was wounded" + RX_POS
"""
! check AI
'user0:aircraft0(0) was heavily wounded at 100.99 200.99'
"""
RX_WAS_HEAVILY_WOUNDED = r"(.+):(.+)\((.+)\) was heavily wounded" + RX_POS
"""
! check AI
'user0:aircraft0 removed at 100.99 200.99'
"""
RX_REMOVED = r"(.+):(.+) removed" + RX_POS



# -*- coding: utf-8 -*-

from twisted.application.internet import TimerService
from twisted.application.service import Service
from twisted.internet import defer
from twisted.python import log

from zope.interface import implementer

from il2ds_middleware.constants import MISSION_STATUS
from il2ds_middleware.interface.service import (IPilotService, IObjectsService,
    IMissionService, )
from il2ds_middleware.parser import EventLogPassthroughParser


class ClientBaseService(Service):

    """Console client is set up manually"""
    client = None


@implementer(IPilotService)
class PilotBaseService(ClientBaseService):

    def user_join(self, info):
        pass

    def user_left(self, info):
        pass

    def user_chat(self, info):
        pass

    def seat_occupied(self, info):
        pass

    def weapons_loaded(self, info):
        pass

    def was_killed(self, info):
        pass

    def was_shot_down(self, info):
        pass

    def selected_army(self, info):
        pass

    def went_to_menu(self, info):
        pass

    def in_flight(self, info):
        pass

    def landed(self, info):
        pass

    def damaged(self, info):
        pass

    def damaged_on_ground(self, info):
        pass

    def turned_wingtip_smokes(self, info):
        pass

    def crashed(self, info):
        pass

    def bailed_out(self, info):
        pass

    def was_captured(self, info):
        pass

    def was_wounded(self, info):
        pass

    def was_heavily_wounded(self, info):
        pass

    def removed(self, info):
        pass


@implementer(IObjectsService)
class ObjectsBaseService(ClientBaseService):

    def was_destroyed(self, info):
        pass


@implementer(IMissionService)
class MissionBaseService(ClientBaseService):

    def on_status_info(self, info):
        pass

    def began(self, info=None):
        pass

    def ended(self, info=None):
        pass


class MissionService(MissionBaseService):

    def __init__(self, log_watcher):
        self.status = None
        self.mission = None
        self.log_watcher = log_watcher

    def on_status_info(self, info):
        status, mission = info
        if status != self.status:
            if self.status == MISSION_STATUS.PLAYING:
                self.ended(info)
            elif status == MISSION_STATUS.PLAYING:
                self.began(info)
        self.status, self.mission = info

    def began(self, info=None):
        self.log_watcher.startService()

    def ended(self, info=None):
        self.log_watcher.stopService()

    def stopService(self):

        def callback(_):
            ClientBaseService.stopService(self)

        return self.log_watcher.stopService().addBoth(callback)


class LogWatchingBaseService(TimerService):

    def __init__(self, log_path, interval=1):
        self.log_file = None
        self.log_path = log_path
        TimerService.__init__(self, interval, self.do_watch)

    def do_watch(self):
        self.log_file.seek(self.log_file.tell())
        for line in self.log_file.readlines():
            self.got_line(line)

    def got_line(self, line):
        pass

    def startService(self):
        if self.log_file is not None:
            return
        try:
            self.log_file = open(self.log_path, 'r')
        except IOError as e:
            log.err("Failed to open events log: {:}.".format(e))
        else:
            self.log_file.seek(self.log_file.tell())
            self.log_file.readlines()
            TimerService.startService(self)

    def stopService(self):
        if self.log_file is None:
            return defer.succeed(None)
        else:
            self.log_file.close()
            self.log_file = None
            return TimerService.stopService(self)


class LogWatchingService(LogWatchingBaseService):

    def __init__(self, log_path, interval=1, parser=None):
        LogWatchingBaseService.__init__(self, log_path, interval)
        self.parser = parser or EventLogPassthroughParser()

    def got_line(self, line):
        idx = line.find(']')
        line = line[idx+1:].lstrip().replace('\n', '')
        self.parser.parse_line(line)



# -*- coding: utf-8 -*-

REQ_SERVER_INFO = "server"

REQ_MISSION_STATUS = "mission"
REQ_MISSION_LOAD = "mission LOAD {0}"
REQ_MISSION_BEGIN = "mission BEGIN"
REQ_MISSION_END = "mission END"
REQ_MISSION_DESTROY = "mission DESTROY"

REQ_CHAT = "chat {0} {1}"
CHAT_ALL = "ALL"
CHAT_USER = "TO {0}"
CHAT_ARMY = "ARMY {0}"



# -*- coding: utf-8 -*-

from twisted.internet.defer import Deferred

from il2ds_middleware.constants import DEVICE_LINK_OPCODE as OPCODE
from il2ds_middleware.ds_emulator.tests.base import BaseEmulatorTestCase


class DeviceLinkTestCase(BaseEmulatorTestCase):

    def test_wrong_format(self):
        d = Deferred()
        self.dl_client.receiver = self._get_unexpecting_line_receiver(d)
        self.dl_client.transport.write("HELLO/WORLD", self.dl_client.address)
        return d

    def test_unknown_command(self):
        d = Deferred()
        self.dl_client.receiver = self._get_unexpecting_line_receiver(d)
        request = ('fake_command', None, )
        self.dl_client.send_request(request)
        return d

    def test_radar_refresh(self):
        d = Deferred()
        self.dl_client.receiver = self._get_unexpecting_line_receiver(d)
        cmd = OPCODE.RADAR_REFRESH.make_command()
        self.dl_client.send_request(cmd)
        return d

    def test_pilot_count(self):
        """
        Scenario:
        1. user0 joins
        2. user1 joins
        3. check count => 0
        4. refresh radar
        5. check count => 0
        6. spawn user0
        7. check count => 0
        8. refresh radar
        9. check count => 1
        10. spawn user1
        11. check count => 1
        12. refresh radar
        13. check count => 2
        14. idle user0
        15. check count => 2
        16. refresh radar
        17. check count => 1
        """
        cmd_radar = OPCODE.RADAR_REFRESH.make_command()
        cmd_count = OPCODE.PILOT_COUNT.make_command()

        pilots = self.service.getServiceNamed('pilots')

        def request_count_with_refresh():
            self.dl_client.send_request(cmd_count)
            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_request(cmd_count)

        def do_test():
            responses = ["A/1002\\0", "A/1002\\0", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_spawn0)

            pilots.join('user0', '192.168.1.2')
            pilots.join('user1', '192.168.1.3')
            request_count_with_refresh()
            return d

        def do_spawn0(_):
            responses = ["A/1002\\0", "A/1002\\1", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_spawn1)

            pilots.spawn('user0')
            request_count_with_refresh()
            return d

        def do_spawn1(_):
            responses = ["A/1002\\1", "A/1002\\2", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_idle)

            pilots.spawn('user1')
            request_count_with_refresh()
            return d

        def do_idle(_):
            responses = ["A/1002\\2", "A/1002\\1", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)

            pilots.idle('user0')
            request_count_with_refresh()
            return d

        return do_test()

    def test_pilot_pos(self):
        """
        Scenario:
        1. check pos with no index
        2. check user0 pos => 0:BADINDEX
        3. user0 joins
        4. user0 spawns at (100, 200, 5)
        5. check user0 pos => 0:user0_0;100;200;5
        6. kill user0
        7. check user0 pos => 0:INVALID
        8. idle user0
        9. check user0 pos => 0:INVALID
        10. refresh radar
        11. check user0 pos => 0:BADINDEX
        12. user1 joins
        13. user0 spawns at (400, 500, 90)
        14. user1 spawns at (700, 800, 50)
        15. refresh radar
        16. check user0 and user1 pos =>
                0:user0_0;400;500;90, 1:user1_1;700;800;50
        """
        cmd_radar = OPCODE.RADAR_REFRESH.make_command()
        cmd_count = OPCODE.PILOT_COUNT.make_command()
        cmd_pos0 = OPCODE.PILOT_POS.make_command(0)
        cmd_pos1 = OPCODE.PILOT_POS.make_command(1)

        pilots = self.service.getServiceNamed('pilots')

        def do_test():
            responses = ["A/1004\\0:BADINDEX", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_join_and_spawn0)

            self.dl_client.send_request(OPCODE.PILOT_POS.make_command())
            self.dl_client.send_request(cmd_pos0)
            return d

        def do_join_and_spawn0(_):
            responses = ["A/1004\\0:user0_0;100;200;5", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_kill0)

            pilots.join('user0', '192.168.1.2')
            pilots.spawn('user0', pos={
                'x': 100, 'y': 200, 'z': 5, })
            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_request(cmd_pos0)
            return d

        def do_kill0(_):
            responses = ["A/1004\\0:INVALID", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_idle0)

            pilots.kill('user0')
            self.dl_client.send_request(cmd_pos0)
            return d

        def do_idle0(_):
            responses = ["A/1004\\0:INVALID", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_idle0_refreshed)

            pilots.idle('user0')
            self.dl_client.send_request(cmd_pos0)
            return d

        def do_idle0_refreshed(_):
            responses = ["A/1004\\0:BADINDEX", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_join1_and_spawn_both)

            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_request(cmd_pos0)
            return d

        def do_join1_and_spawn_both(_):
            responses = [
                "A/1004\\0:user0_0;400;500;90/1004\\1:user1_1;700;800;50", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)

            pilots.join('user1', '192.168.1.3')
            pilots.spawn('user0', pos={
                'x': 400, 'y': 500, 'z': 90, })
            pilots.spawn('user1', pos={
                'x': 700, 'y': 800, 'z': 50, })
            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_requests([cmd_pos0, cmd_pos1, ])
            return d

        return do_test()

    def test_static_count(self):
        """
        Scenario:
        1. check count => 0
        2. spawn 0_Static
        3. spawn 1_Static
        4. check count => 0
        5. refresh radar
        6. check count => 2
        7. destroy 0_Static
        8. check count => 2
        9. refresh radar
        10. check count => 1
        """
        cmd_radar = OPCODE.RADAR_REFRESH.make_command()
        cmd_count = OPCODE.STATIC_COUNT.make_command()

        static = self.service.getServiceNamed('static')

        def do_test():
            responses = ["A/1014\\0", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_spawn)

            self.dl_client.send_request(cmd_count)
            return d

        def do_spawn(_):
            responses = ["A/1014\\0", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_refresh)

            static.spawn('0_Static')
            static.spawn('1_Static')
            self.dl_client.send_request(cmd_count)
            return d

        def do_refresh(_):
            responses = ["A/1014\\2", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_destroy0)

            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_request(cmd_count)
            return d

        def do_destroy0(_):
            responses = ["A/1014\\2", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_refresh_again)

            static.destroy('0_Static')
            self.dl_client.send_request(cmd_count)
            return d

        def do_refresh_again(_):
            responses = ["A/1014\\1", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)

            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_request(cmd_count)
            return d

        return do_test()

    def test_static_pos(self):
        """
        Scenario:
        1. check pos with no index
        2. check both pos => 0:BADINDEX, 1:BADINDEX
        3. 0_Static spawns at (100, 200, 300)
        4. 1_Static spawns at (400, 500, 600)
        5. check both pos => 0:BADINDEX, 1:BADINDEX
        6. refresh radar
        7. check both pos => 0:0_Static;100;200;300, 1:1_Static;400;500;600
        8. destroy 0_Static
        9. check both pos => 0:INVALID, 1:1_Static;400;500;600
        10. refresh radar
        11. check both pos => 0:1_Static;400;500;600, 0:BADINDEX
        """
        cmd_radar = OPCODE.RADAR_REFRESH.make_command()
        cmd_pos0 = OPCODE.STATIC_POS.make_command(0)
        cmd_pos1 = OPCODE.STATIC_POS.make_command(1)
        cmd_pos_both = [cmd_pos0, cmd_pos1, ]

        static = self.service.getServiceNamed('static')

        def do_test():
            responses = ["A/1016\\0:BADINDEX/1016\\1:BADINDEX", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_spawn)

            self.dl_client.send_request(OPCODE.STATIC_POS.make_command())
            self.dl_client.send_requests(cmd_pos_both)
            return d

        def do_spawn(_):
            responses = ["A/1016\\0:BADINDEX/1016\\1:BADINDEX", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_refresh)

            static.spawn('0_Static', pos={
                'x': 100, 'y': 200, 'z': 300, })
            static.spawn('1_Static', pos={
                'x': 400, 'y': 500, 'z': 600, })
            self.dl_client.send_requests(cmd_pos_both)
            return d

        def do_refresh(_):
            responses = [
                "A/1016\\0:0_Static;100;200;300" \
                "/1016\\1:1_Static;400;500;600", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_destroy0)

            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_requests(cmd_pos_both)
            return d

        def do_destroy0(_):
            responses = [
                "A/1016\\0:INVALID" \
                "/1016\\1:1_Static;400;500;600", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)
            d.addCallback(do_refresh_again)

            static.destroy('0_Static')
            self.dl_client.send_requests(cmd_pos_both)
            return d

        def do_refresh_again(_):
            responses = ["A/1016\\0:1_Static;400;500;600/1016\\1:BADINDEX", ]
            d = Deferred()
            self._set_dl_expecting_receiver(responses, d)

            self.dl_client.send_request(cmd_radar)
            self.dl_client.send_requests(cmd_pos_both)
            return d

        return do_test()



# -*- coding: utf-8 -*-

import datetime

from twisted.python import log

from zope.interface import implementer

from il2ds_middleware.interface.parser import ILineParser
from il2ds_middleware.service import LogWatchingBaseService


@implementer(ILineParser)
class LogWatchingService(LogWatchingBaseService):

    receiver = None

    def got_line(self, line):
        try:
            time, data = self.parse_line(line)
        except ValueError:
            log.err("Failed to parse line:\n\t{0}".format(line))
        else:
            if self.receiver is not None:
                self.receiver(data)

    def parse_line(self, line):
        idx = line.find(']')
        time_raw = line[1:idx]
        format = "%I:%M:%S %p"
        try:
            time = datetime.datetime.strptime(time_raw, format)
        except ValueError:
            format = "%b %d, %Y " + format
            time = datetime.datetime.strptime(time_raw, format)
        finally:
            data = line[idx + 1:].lstrip()
            return time, data



# -*- coding: utf-8 -*-

import datetime
import os

from twisted.application import internet
from twisted.application.service import Service, MultiService
from twisted.python import log
from zope.interface import implementer

from il2ds_middleware.constants import (DEVICE_LINK_OPCODE as OPCODE,
    MISSION_STATUS, PILOT_STATE, OBJECT_STATE, )
from il2ds_middleware.interface.parser import ILineParser
from il2ds_middleware.ds_emulator.constants import (LONG_OPERATION_DURATION,
    LONG_OPERATION_CMD, )
from il2ds_middleware.ds_emulator.interfaces import (IPilotService,
    IMissionService, IStaticObjectService, IDeviceLinkService, IEventLogger, )


@implementer(ILineParser)
class _CommonServiceMixin():

    evt_log = None
    parent = None
    client = None

    def send(self, line):
        if self.client:
            self.client.message(line)
        elif self.parent:
            self.parent.send(line)


class RootService(MultiService, _CommonServiceMixin):
    """
    Top-level service.
    """
    lop_duration = LONG_OPERATION_DURATION

    def __init__(self, log_path=None):
        MultiService.__init__(self)
        self.evt_log = EventLoggingService(log_path)
        self.user_command_id = 0
        self.set_server_info()
        self._init_children()

    def _init_children(self):
        """
        Initialize children services.
        """
        pilots = PilotService()
        static = StaticService()
        dl = DeviceLinkService()
        missions = MissionService()

        dl.pilot_srvc = pilots
        dl.static_srvc = static
        missions.dl_srvc = dl

        for service in [pilots, static, missions, dl, ]:
            service.setServiceParent(self)
            service.evt_log = self.evt_log

    def startService(self):
        MultiService.startService(self)

    def stopService(self):
        self.evt_log.setServiceParent(self)
        return MultiService.stopService(self)

    def parse_line(self, line):
        result = False
        for service in self.services:
            if not ILineParser.providedBy(service):
                continue
            result = service.parse_line(line)
            if result:
                break
        if not result and not self._parse(line):
            self.send("Command not found: " + line)
        return True

    def _parse(self, line):
        while True:
            if line == 'server':
                self._server_info()
                break
            if line == LONG_OPERATION_CMD:
                self._long_operation()
                break
            if self._chat(line):
                break
            return False
        return True

    def _server_info(self):
        response = [
            "Type: Local server",
            "Name: {0}".format(self.info['name'] or "Server"),
            "Description: {0}".format(self.info['description'] or ""),
        ]
        for line in response:
            self.send(line)

    def _long_operation(self):
        import time
        time.sleep(self.lop_duration)

    def _chat(self, line):
        if not line.startswith("chat"):
            return False
        idx = line.find('ALL')
        if idx < 0:
            idx = line.find('TO')
        if idx < 0:
            idx = line.find('ARMY')
        if idx < 0:
            return False
        msg = line[4:idx].strip()
        self.send("Chat: Server: \t{0}".format(msg))
        return True

    def set_server_info(self, name="", description=""):
        self.info = {
            'name': name,
            'description': description,
        }

    def manual_input(self, line):
        self.send(line)
        self.parse_line(line)

        self.user_command_id += 1
        self.send("<consoleN><{0}>".format(self.user_command_id))


@implementer(IPilotService)
class PilotService(Service, _CommonServiceMixin):

    name = "pilots"
    channel = 1
    channel_inc = 2
    port = 21000

    def __init__(self):
        self.pilots = {}

    def parse_line(self, line):
        while True:
            if line.startswith("kick"):
                self._kick(callsign=line[4:].strip())
                break
            return False
        return True

    def join(self, callsign, ip):

        def create_pilot():
            pilot = {
                'ip': ip,
                'channel': self.channel,
                'state': PILOT_STATE.IDLE,
                'army': "None",
            }
            self.channel += self.channel_inc
            return pilot

        pilot = create_pilot()
        self.pilots[callsign] = pilot

        self.send(
            "socket channel '{0}' start creating: ip {1}:{2}".format(
                pilot['channel'], pilot['ip'], self.port))
        self.send("Chat: --- {0} joins the game.".format(callsign))
        self.send(
            "socket channel '{0}', ip {1}:{2}, {3}, "
            "is complete created.".format(
                pilot['channel'], pilot['ip'], self.port, callsign))
        self.evt_log.enlog("{0} has connected".format(callsign))

    def leave(self, callsign):
        self._leave(callsign)

    def _kick(self, callsign):
        self._leave(callsign, reason="You have been kicked from the server.")

    def _leave(self, callsign, reason=None):
        pilot = self.pilots.get(callsign)
        if pilot is None:
            log.err("Pilot with callsign \"{0}\" not found.".format(callsign))
            return
        del self.pilots[callsign]

        line = "socketConnection with {0}:{1} on channel {2} lost.  " \
            "Reason: ".format(pilot['ip'], self.port, pilot['channel'])
        if reason:
            line += reason
        self.send(line)
        self.send("Chat: --- {0} has left the game.".format(
            callsign))
        self.evt_log.enlog("{0} has disconnected".format(callsign))

    def idle(self, callsign):
        pilot = self.pilots.get(callsign)
        if pilot is not None:
            pilot['state'] = PILOT_STATE.IDLE
            self.evt_log.enlog("{0} entered refly menu".format(callsign))

    def spawn(self, callsign, craft=None, pos=None):
        pilot = self.pilots.get(callsign)
        if pilot is not None:
            pilot['state'] = PILOT_STATE.SPAWNED
            pilot['pos'] = pos or {
                'x': 0, 'y': 0, 'z': 0, }
            pilot['craft'] = craft or {
                'name': "A6M2-21",
                'weapons': "1xdt",
                'fuel': "100",
            }
            self.evt_log.enlog(
                "{0}:{1}(0) seat occupied by {0} at {2} {3}".format(
                    callsign, pilot['craft']['name'],
                    pilot['pos']['x'], pilot['pos']['y']))
            self.evt_log.enlog(
                "{0}:{1} loaded weapons '{2}' fuel {3}%".format(
                    callsign, pilot['craft']['name'],
                    pilot['craft']['weapons'], pilot['craft']['fuel']))

    def kill(self, callsign):
        pilot = self.pilots.get(callsign)
        if pilot is not None:
            pilot['state'] = PILOT_STATE.DEAD
            self.evt_log.enlog(
                "{0}:{1}(0) was killed at {2} {3}".format(
                    callsign, pilot['craft']['name'],
                    pilot['pos']['x'], pilot['pos']['y']))

    def get_active(self):
        return [x for x in self.pilots.keys()
            if self.pilots[x]['state'] != PILOT_STATE.IDLE]

    def stopService(self):
        self.pilots = None
        return Service.stopService(self)


@implementer(IMissionService)
class MissionService(Service, _CommonServiceMixin):

    name = "missions"
    status = MISSION_STATUS.NOT_LOADED
    mission = None
    dl_srvc = None

    def parse_line(self, line):
        if not line.startswith("mission"):
            return False
        cmd = line[7:].strip()
        while True:
            if not cmd:
                self._send_status()
                break
            elif cmd.startswith("LOAD"):
                self.load(mission=cmd[4:].lstrip())
                break
            elif cmd == "BEGIN":
                self.begin()
                break
            elif cmd == "END":
                self.end()
                break
            elif cmd == "DESTROY":
                self.destroy()
                break
            return False
        return True

    def load(self, mission):
        self.mission = mission
        self.send("Loading mission {0}...".format(self.mission))
        self.send("Load bridges")
        self.send("Load static objects")
        self.send("##### House without collision "
            "(3do/Tree/Tree2.sim)")
        self.send("##### House without collision "
            "(3do/Buildings/Port/Floor/live.sim)")
        self.send("##### House without collision "
            "(3do/Buildings/Port/BaseSegment/live.sim)")
        self.status = MISSION_STATUS.LOADED
        self._send_status()

    def begin(self):
        if self.status == MISSION_STATUS.NOT_LOADED:
            self._mission_not_loaded()
        else:
            self.evt_log.start_log()
            self.status = MISSION_STATUS.PLAYING
            self._send_status()
            self.evt_log.enlog("Mission: {0} is Playing".format(self.mission))
            self.evt_log.enlog("Mission BEGIN")

    def end(self):
        if self.status == MISSION_STATUS.NOT_LOADED:
            self._mission_not_loaded()
        else:
            self.status = MISSION_STATUS.LOADED
            self._send_status()
            self.evt_log.enlog("Mission END")
            self.evt_log.stop_log()

    def destroy(self):
        if self.status == MISSION_STATUS.NOT_LOADED:
            self._mission_not_loaded()
        else:
            self.status = MISSION_STATUS.NOT_LOADED
            self.mission = None
            if self.dl_srvc:
                self.dl_srvc.forget_everything()

    def _mission_not_loaded(self):
        self.send("ERROR mission: Mission NOT loaded")

    def _send_status(self):
        if self.status == MISSION_STATUS.NOT_LOADED:
            self.send("Mission NOT loaded")
        elif self.status == MISSION_STATUS.LOADED:
            self.send("Mission: {0} is Loaded".format(self.mission))
        elif self.status == MISSION_STATUS.PLAYING:
            self.send("Mission: {0} is Playing".format(self.mission))

    def stopService(self):
        self.mission = None
        return Service.stopService(self)


@implementer(IStaticObjectService)
class StaticService(Service):

    name = "static"
    objects = None

    def __init__(self):
        self.objects = {}

    def spawn(self, name, pos=None):
        self.objects[name] = {
            'pos': pos or {
                'x': 0, 'y': 0, 'z': 0, },
            'state': OBJECT_STATE.ALIVE,
        }

    def destroy(self, name, attacker_name='landscape'):
        obj = self.objects[name]
        obj['state'] = OBJECT_STATE.DESTROYED
        self.evt_log.enlog("{0} destroyed by {1} at {2} {3}".format(
            name, attacker_name, obj['pos']['x'], obj['pos']['y']))

    def get_active(self):
        return [x for x in self.objects.keys()
            if self.objects[x]['state'] != OBJECT_STATE.DESTROYED]


@implementer(IDeviceLinkService)
class DeviceLinkService(Service):

    name = "dl"
    lop_duration = LONG_OPERATION_DURATION
    pilot_srvc = None
    static_srvc = None

    def __init__(self):
        self.forget_everything()

    def forget_everything(self):
        self.known_air = []
        self.known_static = []

    def refresh_radar(self):
        self.known_air = self.pilot_srvc.get_active()
        self.known_static = self.static_srvc.get_active()

    def pilot_count(self):
        result = len(self.known_air)
        return OPCODE.PILOT_COUNT.make_command(result)

    def pilot_pos(self, arg):
        data = self._pos(
            known_container=self.known_air,
            primary_container=self.pilot_srvc.pilots,
            invalid_states=[PILOT_STATE.IDLE, PILOT_STATE.DEAD, ],
            idx=arg, idx_append=True)
        return OPCODE.PILOT_POS.make_command(data) if data else None

    def static_count(self):
        result = len(self.known_static)
        return OPCODE.STATIC_COUNT.make_command(result)

    def static_pos(self, arg):
        data = self._pos(
            known_container=self.known_static,
            primary_container=self.static_srvc.objects,
            invalid_states=[OBJECT_STATE.DESTROYED, ],
            idx=arg)
        return OPCODE.STATIC_POS.make_command(data) if data else None

    def _pos(self, known_container, primary_container, invalid_states,
            idx, idx_append=False):
        if idx is None:
            return None
        try:
            key = known_container[int(idx)]
        except Exception:
            data = 'BADINDEX'
        else:
            handler = primary_container[key]
            if handler['state'] in invalid_states:
                data = 'INVALID'
            else:
                if idx_append:
                    key = "{:}_{:}".format(key, idx)
                pos = handler['pos']
                data = ';'.join([str(_)
                    for _ in [key, pos['x'], pos['y'], pos['z'], ]])
        finally:
            return ':'.join([idx, data, ])

    def long_operation(self):
        import time
        time.sleep(self.lop_duration)
        return (LONG_OPERATION_CMD, None, )


@implementer(IEventLogger)
class EventLoggingService(Service):

    def __init__(self, log_path=None, keep_log=True):
        self.log_path = log_path
        self.keep_log = keep_log
        self.log_file = None
        self.last_evt_time = None

    def enlog(self, line):
        if self.log_file is not None:
            self._do_log(line)
        else:
            log.msg("Logging event into nowhere: \"{0}\"".format(line))

    def _do_log(self, line):
        evt_time = datetime.datetime.now()
        timestamp = self._get_formated_time(evt_time)
        _line = "[{0}] {1}\n".format(timestamp, line)
        self.log_file.write(_line)
        self.last_evt_time = evt_time

    def _get_formated_time(self, timestamp):
        """
        We do not need leading zero before hours, so we will replace it
        if it is present. To find it we use symbol '-'.
        """
        format = "-%I:%M:%S %p"
        if self._day_differs(timestamp):
            format = "%b %d, %Y " + format
        return timestamp.strftime(format).replace('-0', '').replace('-', '')

    def _day_differs(self, timestamp):
        return timestamp.day != self.last_evt_time.day \
            if self.last_evt_time is not None else True

    def start_log(self):
        self.stop_log()
        if self.log_path is not None:
            self.log_file = open(self.log_path, 'a' if self.keep_log else 'w')

    def stop_log(self):
        if self.log_file is not None:
            self.log_file.close()
            self.log_file = None

    def stopService(self):
        self.stop_log()
        if self.log_path is not None and os.path.isfile(self.log_path):
            os.remove(self.log_path)
        return Service.stopService(self)



# -*- coding: utf-8 -*-

from zope.interface import Interface

from twisted.application.service import IService


class IPilotService(IService):

    def join(self, callsign, ip):
        """Emulate joining pilot to server.

        :param callsign: pilot's callsign.
        :type callsign: str.

        :param ip: remote pilot's ip address.
        :type ip: str.
        """

    def leave(self, callsign):
        """Emulate disconnecting pilot from server.

        :param callsign: pilot's callsign.
        :type callsign: str.
        """

    def kick(self, callsign):
        """Emulate pilot kicking from server.

        :param callsign: pilot's callsign.
        :type callsign: str.
        """

    def idle(self, callsign):
        """Emulate setting pilot's state to IDLE. Usually it means going to
        the refly menu.

        :param callsign: pilot's callsign.
        :type callsign: str.
        """

    def spawn(self, callsign, craft, pos):
        """Emulate spawning pilot at map.

        :param callsign: pilot's callsign.
        :type callsign: str.

        :param craft: description of craft: name, weapons, fuel.
        :type craft: dict.

        :param pos: position on map: z, y, z.
        :type pos: dict.
        """

    def kill(self, callsign):
        """Emulate killing pilot.

        :param callsign: pilot's callsign.
        :type callsign: str.
        """

    def get_active(self):
        """Get pilots which are not in IDLE state.

        :returns: str list -- list of callsigns.
        """


class IMissionService(IService):

    def load(self, mission):
        """Emulate loading mission to server.

        :param mission: path to mission's file relatively to server's root.
        :type mission: str.
        """

    def begin(self):
        """Emulate beginning of loaded mission."""

    def end(self):
        """Emulate ending of running mission."""

    def destroy(self):
        """Emulate destroying of running or loaded mission."""


class IStaticObjectService(IService):

    def spawn(self, name, pos):
        """Emulate spawning object at map.

        :param name: objects map name.
        :type name: str.

        :param pos: position on map: z, y, z.
        :type pos: dict.
        """

    def destroy(self, name, attacker_name):
        """Emulate destroying object by attaker.

        :param name: objects map name.
        :type name: str.

        :param attacker_name: name of attaker.
        :type attacker_name: str.
        """

    def get_active(self):
        """Get objects which are not destriyed.

        :returns: str list -- list of names.
        """


class IDeviceLinkService(IService):

    def forget_everything(self):
        """Forget any knows pilots, ojects, etc.
        """

    def got_requests(self, requests, address, peer):
        """Process incoming requests.

        :param requests: list of requests to process.
        :type requests: list.

        :param address: (host, port) of request initiator.
        :type address: tuple.

        :param peer: protocol instance used for sending answers.
        :type peer: DeviceLinkProtocol.
        """


class IEventLogger(IService):

    def enlog(self, line):
        """Write line to log.

        :param line: line to write to log.
        :type line: str.
        """

    def start_log(self):
        """Start logging: open file for writing or appending, etc.
        """

    def stop_log(self):
        """Stop logging: close logging file, etc.
        """



# -*- coding: utf-8 -*-

from twisted.internet.defer import Deferred
from twisted.internet.protocol import ServerFactory
from twisted.protocols.basic import LineReceiver
from twisted.python import log

from il2ds_middleware.constants import DEVICE_LINK_OPCODE as OPCODE
from il2ds_middleware.protocol import DeviceLinkProtocol
from il2ds_middleware.ds_emulator.constants import LONG_OPERATION_CMD


class ConsoleServer(LineReceiver):

    service = None

    def connectionMade(self):
        self.factory.client_connected(self)

    def connectionLost(self, reason):
        self.factory.client_left(self)

    def lineReceived(self, line):
        if self.service is not None:
            self.service.parse_line(line)

    def message(self, message):
        self.sendLine(message + '\\n')


class ConsoleServerFactory(ServerFactory):

    protocol = ConsoleServer

    def __init__(self):
        self.on_connected = Deferred()
        self.on_connection_lost = Deferred()

    def client_connected(self, client):
        if self.on_connected is not None:
            d, self.on_connected = self.on_connected, None
            d.callback(client)

    def client_left(self, client):
        if self.on_connection_lost is not None:
            d, self.on_connection_lost = self.on_connection_lost, None
            d.callback(client)


class DeviceLinkServerProtocol(DeviceLinkProtocol):

    service = None

    def requests_received(self, requests, address):
        answers = []
        for request in requests:
            cmd, arg = request
            answer = None
            try:
                opcode = OPCODE.lookupByValue(cmd)
            except ValueError as e:
                if cmd == LONG_OPERATION_CMD:
                    answers.append(self.service.long_operation())
                else:
                    log.err("Unknown command: {0}".format(cmd))
            else:
                if opcode == OPCODE.RADAR_REFRESH:
                    self.service.refresh_radar()
                elif opcode == OPCODE.PILOT_COUNT:
                    answer = self.service.pilot_count()
                elif opcode == OPCODE.PILOT_POS:
                    answer = self.service.pilot_pos(arg)
                elif opcode == OPCODE.STATIC_COUNT:
                    answer = self.service.static_count()
                elif opcode == OPCODE.STATIC_POS:
                    answer = self.service.static_pos(arg)
                if answer is not None:
                    answers.append(answer)
        if answers:
            self.send_answers(answers, address)



# -*- coding: utf-8 -*-

from twisted.python.constants import (ValueConstant, Values, NamedConstant,
    Names, )


class DeviceLinkValueConstant(ValueConstant):

    def make_command(self, arg=None):
        return (self.value, arg, )


class DEVICE_LINK_OPCODE(Values):
    """
    Constants representing operation codes for DeviceLink interface.
    """
    RADAR_REFRESH = DeviceLinkValueConstant("1001")
    PILOT_COUNT = DeviceLinkValueConstant("1002")
    PILOT_POS = DeviceLinkValueConstant("1004")
    STATIC_COUNT = DeviceLinkValueConstant("1014")
    STATIC_POS = DeviceLinkValueConstant("1016")


DEVICE_LINK_PREFIXES = {
    'answer': 'A',
    'request': 'R',
}

DEVICE_LINK_CMD_SEPARATOR = '/'
DEVICE_LINK_ARGS_SEPARATOR = '\\'
DEVICE_LINK_CMD_GROUP_MAX_SIZE = 40


class MISSION_STATUS(Names):
    """
    Constants representing various mission status codes.
    """
    NOT_LOADED = NamedConstant()
    LOADED = NamedConstant()
    PLAYING = NamedConstant()


class PILOT_STATE(Names):
    """
    Constants representing pilot states.
    """
    IDLE = NamedConstant()
    SPAWNED = NamedConstant()
    IN_FLIGHT = NamedConstant()
    DEAD = NamedConstant()


class OBJECT_STATE(Names):
    """
    Constants representing object states.
    """
    ALIVE = NamedConstant()
    DESTROYED = NamedConstant()


REQUEST_TIMEOUT = 0.1
REQUEST_MISSION_LOAD_TIMEOUT = 30


class PILOT_LEAVE_REASON(Names):
    """
    Constants representing the reason why a pilot has left.
    """
    DISCONNECTED = NamedConstant()
    KICKED = NamedConstant()


CHAT_MAX_LENGTH = 80



# -*- coding: utf-8 -*-

from twisted.internet.defer import Deferred
from twisted.internet.protocol import ClientFactory
from twisted.protocols.basic import LineReceiver

from il2ds_middleware.protocol import DeviceLinkProtocol


class ConsoleClient(LineReceiver):

    def connectionMade(self):
        self.factory.client_joined(self)

    def connectionLost(self, reason):
        self.factory.client_left(self)

    def lineReceived(self, line):
        self.got_line(line)

    def got_line(self, line):
        if self.factory.receiver is not None:
            self.factory.receiver(line)


class ConsoleClientFactory(ClientFactory):

    protocol = ConsoleClient
    receiver = None

    def __init__(self, parser=None):
        self.clients = []
        self.on_connecting = Deferred()
        self.on_connection_lost = Deferred()

    def client_joined(self, client):
        if self.on_connecting:
            d, self.on_connecting = self.on_connecting, None
            d.callback(client)
        self.clients.append(client)

    def client_left(self, client):
        if self.on_connection_lost:
            d, self.on_connection_lost = self.on_connection_lost, None
            d.callback(client)
        self.clients.remove(client)


class DeviceLinkClient(DeviceLinkProtocol):

    receiver = None

    def answers_received(self, answers, address):
        if self.receiver is not None:
            line = "A/" + self._format(answers)
            self.receiver(line)



# -*- coding: utf-8 -*-

from twisted.internet import defer
from twisted.internet.error import ConnectionDone
from twisted.internet.protocol import (ClientFactory, DatagramProtocol, )
from twisted.protocols.basic import LineOnlyReceiver
from twisted.python import log

from il2ds_middleware.constants import (DEVICE_LINK_OPCODE as DL_OPCODE,
    DEVICE_LINK_PREFIXES, DEVICE_LINK_CMD_SEPARATOR as DL_CMD_SEP,
    DEVICE_LINK_ARGS_SEPARATOR as DL_ARGS_SEP, DEVICE_LINK_CMD_GROUP_MAX_SIZE,
    REQUEST_TIMEOUT, REQUEST_MISSION_LOAD_TIMEOUT, CHAT_MAX_LENGTH, )
from il2ds_middleware.parser import (ConsolePassthroughParser,
    DeviceLinkPassthroughParser, )
from il2ds_middleware.requests import (
    REQ_SERVER_INFO, REQ_MISSION_STATUS, REQ_MISSION_LOAD, REQ_MISSION_BEGIN,
    REQ_MISSION_END, REQ_MISSION_DESTROY, REQ_CHAT, CHAT_ALL, CHAT_USER,
    CHAT_ARMY, )


class ConsoleClient(LineOnlyReceiver):

    """Parser and timeout_value are set up by factory"""
    parser = None
    timeout_value = None

    def __init__(self):
        self._request_id = 0
        self._responce_id = None
        # { request_id: ([results], deferred, timeout, ), }
        self._requests = {}

    def connectionMade(self):
        self.factory.clientConnectionMade(self)

    def _generate_request_id(self):
        self._request_id += 1
        return self._request_id

    def _make_request(self, d, timeout_value):
        rid = self._generate_request_id()

        def on_timeout(_):
            del self._requests[rid]
            defer.timeout(d)

        from twisted.internet import reactor
        timeout = reactor.callLater(
            timeout_value or self.timeout_value, on_timeout, None)
        return rid, ([], d, timeout, )

    def _send_request(self, line, timeout_value=None):
        d = defer.Deferred()
        rid, request = self._make_request(d, timeout_value)
        self._requests[rid] = request

        wrapper = "rid|{0}".format(rid)
        self.sendLine(wrapper)
        self.sendLine(line)
        self.sendLine(wrapper)
        return d

    def lineReceived(self, line):
        if line.startswith("<consoleN><"):
            return
        self.got_line(line.replace("\\n", '').replace("\\u0020", ' '))

    def got_line(self, line):
        if line.startswith("Command not found: rid"):
            self._process_responce_id(line)
        elif self._responce_id:
            self._requests[self._responce_id][0].append(line)
        else:
            self.parser.parse_line(line)

    def _process_responce_id(self, line):
        try:
            rid = int(line.split('|')[1])
        except IndexError:
            log.err("RID format is malformed in \"{0}\"".format(line))
        except ValueError:
            log.err("Could not get RID value from \"{0}\"".format(line))
        else:
            if rid not in self._requests:
                log.err("Unexpected RID: {0}".format(rid))
                return
            if self._responce_id is None:
                # start request processing
                self._responce_id = rid
            elif self._responce_id == rid:
                # end request processing, send results to callback
                self._responce_id = None
                results, d, timeout = self._requests[rid]
                timeout.cancel()
                del self._requests[rid]
                d.callback(results)

    def server_info(self):
        d = self._send_request(REQ_SERVER_INFO)
        d.addCallback(self.parser.server_info)
        return d

    def mission_status(self, *args):
        d = self._send_request(REQ_MISSION_STATUS)
        d.addCallback(self.parser.mission_status)
        return d

    def mission_load(self, mission):
        d = self._send_request(
            REQ_MISSION_LOAD.format(mission), REQUEST_MISSION_LOAD_TIMEOUT)
        d.addCallback(self.parser.mission_status)
        return d

    def mission_begin(self):
        d = self._send_request(REQ_MISSION_BEGIN)
        d.addCallback(self.parser.mission_status)
        return d

    def mission_end(self):
        d = self._send_request(REQ_MISSION_END)
        d.addCallback(lambda _: self.mission_status())
        return d

    def mission_destroy(self):
        d = self._send_request(REQ_MISSION_DESTROY)
        d.addCallback(lambda _: self.mission_status())
        return d

    def chat_all(self, message):
        self._chat(message, CHAT_ALL)

    def chat_user(self, message, callsign):
        self._chat(message, CHAT_USER.format(callsign))

    def chat_army(self, message, army):
        self._chat(message, CHAT_ARMY.format(army))

    def _chat(self, message, suffix):
        last = 0
        total = len(message)
        while last < total:
            step = min(CHAT_MAX_LENGTH, total-last)
            chunk = message[last:last+step]
            self.sendLine(
                REQ_CHAT.format(chunk.encode('unicode-escape'), suffix))
            last += step


class ConsoleClientFactory(ClientFactory):

    protocol = ConsoleClient

    def __init__(self, parser=None, timeout_value=REQUEST_TIMEOUT):
        self._client = None
        self.parser = parser
        self.timeout_value = timeout_value
        self.on_connecting = defer.Deferred()
        self.on_connection_lost = defer.Deferred()

    def buildProtocol(self, addr):
        self._client = ClientFactory.buildProtocol(self, addr)
        parser, self.parser = self.parser or ConsolePassthroughParser(), None
        tv, self.timeout_value = self.timeout_value, None
        self._client.parser = parser
        self._client.timeout_value = tv
        return self._client

    def clientConnectionMade(self, client):
        if self.on_connecting is not None:
            d, self.on_connecting = self.on_connecting, None
            d.callback(client)

    def clientConnectionFailed(self, connector, reason):
        if self.on_connecting is not None:
            d, self.on_connecting = self.on_connecting, None
            d.errback(reason)

    def clientConnectionLost(self, connector, reason):
        if self.on_connection_lost is not None:
            d, self.on_connection_lost = self.on_connection_lost, None
            if isinstance(reason.value, ConnectionDone):
                d.callback(None)
            else:
                d.errback(reason)


class DeviceLinkProtocol(DatagramProtocol):

    def __init__(self, address=None, parser=None):
        self.address = address
        self.on_start = defer.Deferred()

    def startProtocol(self):
        if self.on_start is not None:
            d, self.on_start = self.on_start, None
            d.callback(None)

    def datagramReceived(self, data, (host, port)):
        if data.startswith(DEVICE_LINK_PREFIXES['answer']):
            processor = self.answers_received
        elif data.startswith(DEVICE_LINK_PREFIXES['request']):
            processor = self.requests_received
        else:
            log.err("Malformed data from {0}: \"{1}\"")
            return
        prepared_data = data[1:].strip(DL_CMD_SEP)
        payloads = self._parse(prepared_data)
        processor(payloads, (host, port))

    def _parse(self, data):
        results = []
        for chunk in data.split(DL_CMD_SEP):
            command = chunk.split(DL_ARGS_SEP)
            arg = command[1:]
            results.append((command[0], arg[0] if arg else None, ))
        return results

    def answers_received(self, answers, address):
        pass

    def requests_received(self, requests, address):
        pass

    def send_request(self, request, address=None):
        self.send_requests([request, ], address)

    def send_requests(self, requests, address=None):
        self._send_payloads(DEVICE_LINK_PREFIXES['request'], requests, address)

    def send_answer(self, answer, address=None):
        self.send_answers([answer, ], address)

    def send_answers(self, answers, address=None):
        self._send_payloads(DEVICE_LINK_PREFIXES['answer'], answers, address)

    def _send_payloads(self, prefix, payloads, address=None):
        data = DL_CMD_SEP.join(
            [prefix, self._format(payloads), ])
        self.transport.write(data, address or self.address)

    def _format(self, payloads):
        chunks = []
        for payload in payloads:
            cmd, arg = payload
            chunks.append(
                DL_ARGS_SEP.join([cmd, str(arg)]) if arg is not None else cmd)
        return DL_CMD_SEP.join(chunks)


class DeviceLinkClient(DeviceLinkProtocol):

    cmd_group_max_size = DEVICE_LINK_CMD_GROUP_MAX_SIZE

    def __init__(self, address, parser=None, timeout_value=REQUEST_TIMEOUT):
        self.parser = parser or DeviceLinkPassthroughParser()
        self.timeout_value = timeout_value
        DeviceLinkProtocol.__init__(self, address)
        # [ (opcode, deferred, timeout), ]
        self._requests = []

    def answers_received(self, answers, address):
        if address == self.address:
            for answer in answers:
                self._answer_received(answer)

    def _answer_received(self, answer):
        cmd, arg = answer
        for request in self._requests:
            opcode, d, timeout = request
            if opcode == cmd:
                timeout.cancel()
                self._requests.remove(request)
                d.callback(arg)
                break

    def _make_request(self, command, d, timeout_value=None):

        def on_timeout(_):
            self._requests.remove(request)
            defer.timeout(d)

        from twisted.internet import reactor
        timeout = reactor.callLater(
            timeout_value or self.timeout_value, on_timeout, None)
        request = (command[0], d, timeout, )
        return request

    def _deferred_request(self, command, timeout_value=None):
        d = defer.Deferred()
        self._requests.append(
            self._make_request(command, d, timeout_value))
        self.send_request(command)
        return d

    @defer.inlineCallbacks
    def _deferred_requests(self, commands, timeout_value=None):

        def on_results(results):
            all_results.extend(results)

        all_results = []
        step = self.cmd_group_max_size
        count = len(commands)
        for i in xrange((count/step)+1):
            start = i*step
            group = commands[start:start+min((count-start), step)]
            dlist = []
            for cmd in group:
                d = defer.Deferred()
                self._requests.append(
                    self._make_request(cmd, d, timeout_value))
                dlist.append(d)
            self.send_requests(group)
            yield defer.gatherResults(dlist).addCallback(on_results)
        defer.returnValue(all_results)

    def refresh_radar(self):
        self.send_request(DL_OPCODE.RADAR_REFRESH.make_command())
        return defer.succeed(None)

    def pilot_count(self):
        d = self._deferred_request(DL_OPCODE.PILOT_COUNT.make_command())
        d.addCallback(self.parser.pilot_count)
        return d

    def pilot_pos(self, index):
        d = self._deferred_request(DL_OPCODE.PILOT_POS.make_command(index))
        d.addCallback(self.parser.pilot_pos)
        return d

    def all_pilots_pos(self):
        d = self._all_pos(self.pilot_count, DL_OPCODE.PILOT_POS)
        d.addCallback(self.parser.all_pilots_pos)
        return d

    def static_count(self):
        d = self._deferred_request(DL_OPCODE.STATIC_COUNT.make_command())
        d.addCallback(self.parser.static_count)
        return d

    def static_pos(self, index):
        d = self._deferred_request(DL_OPCODE.STATIC_POS.make_command(index))
        d.addCallback(self.parser.static_pos)
        return d

    def all_static_pos(self):
        d = self._all_pos(self.static_count, DL_OPCODE.STATIC_POS)
        d.addCallback(self.parser.all_static_pos)
        return d

    def _all_pos(self, get_count, pos_opcode):

        def on_count(result):
            count = int(result)
            if not count:
                return []
            return self._deferred_requests([
                pos_opcode.make_command(i) for i in xrange(count)])

        return get_count().addCallback(on_count)



# -*- coding: utf-8 -*-

from twisted.internet import defer
from twisted.trial.unittest import TestCase

from il2ds_middleware.parser import (ConsolePassthroughParser,
    DeviceLinkPassthroughParser, )
from il2ds_middleware.protocol import ConsoleClientFactory, DeviceLinkClient

from il2ds_middleware.ds_emulator.service import RootService as DSService
from il2ds_middleware.ds_emulator.protocol import (ConsoleServerFactory,
    DeviceLinkServerProtocol, )


class BaseTestCase(TestCase):

    console_client_factory_class = None
    console_client_parser_class = ConsolePassthroughParser
    dl_client_class = None
    dl_client_parser_class = DeviceLinkPassthroughParser
    log_watcher_class = None

    console_server_host = "127.0.0.1"
    console_server_port = 0
    device_link_server_host = "127.0.0.1"
    device_link_server_port = 0
    log_path = None

    log_watcher_interval = 0.1
    timeout_value = 0.05

    def setUp(self):
        self.service = DSService(self.log_path)
        self.service.startService()
        self._set_log_watcher()
        self._listen_server()
        return self._connect_client()

    def _set_log_watcher(self):
        self.log_watcher = self.log_watcher_class(
            self.log_path, self.log_watcher_interval
        ) if self.log_watcher_class else None

    def _listen_server(self):
        self._listen_console_server()
        self._listen_device_link_server()

    def _listen_console_server(self):

        def on_connected(client):
            self.console_server = client
            client.service = self.service
            self.service.client = client
            return client

        def on_disconnected(client):
            self.console_server = None
            client.service = None
            self.service.client = None
            return client

        self.console_server_factory = ConsoleServerFactory()
        self.console_server_factory.on_connected.addCallback(on_connected)
        self.console_server_factory.on_connection_lost.addBoth(
            on_disconnected)

        from twisted.internet import reactor
        self.console_server_listener = reactor.listenTCP(
            self.console_server_port, self.console_server_factory,
            interface=self.console_server_host)

    def _listen_device_link_server(self):
        self.dl_server = DeviceLinkServerProtocol()
        self.dl_server.service = self.service.getServiceNamed('dl')

        from twisted.internet import reactor
        self.dl_server_listener = reactor.listenUDP(
            self.device_link_server_port, self.dl_server,
            interface=self.device_link_server_host)

    def _connect_client(self):
        self._connect_device_link_client()
        return self._connect_console_client()

    def _connect_console_client(self):

        def on_connected(client):
            self.console_client = client

        def on_disconnected(client):
            self.console_client = None

        if self.console_client_factory_class is None:
            self.console_client_connector = None
            return defer.succeed(_)

        parser = self.console_client_parser_class()
        self.console_client_factory = self.console_client_factory_class(parser)
        self.console_client_factory.on_connecting.addCallback(on_connected)
        self.console_client_factory.on_connection_lost.addBoth(on_disconnected)

        host, port = self.console_client_host_for_client

        from twisted.internet import reactor
        self.console_client_connector = reactor.connectTCP(
            host, port, self.console_client_factory)

        return self.console_client_factory.on_connecting

    def _connect_device_link_client(self):
        if self.dl_client_class is None:
            self.dl_client_connector = None
            return

        parser = (self.dl_client_parser_class()
            if self.dl_client_parser_class else None)
        self.dl_client = self.dl_client_class(
            self.device_link_host_for_client, parser)

        from twisted.internet import reactor
        self.dl_client_connector = reactor.listenUDP(0, self.dl_client)

    def tearDown(self):
        dlist = [self.service.stopService(), ]
        if self.log_watcher is not None:
            dlist.append(self.log_watcher.stopService())
        dlist.extend(self._stop_server())
        dlist.extend(self._stop_client())
        return defer.gatherResults(dlist)

    def _stop_server(self):
        results = []
        results.extend(self._stop_console_server())
        results.append(self._stop_device_link_server())
        return results

    def _stop_console_server(self):
        results = [
            defer.maybeDeferred(self.console_server_listener.stopListening), ]
        if self.console_client_connector is not None:
            d = self.console_server_factory.on_connection_lost
            if d is not None:
                results.append(d)
        return results

    def _stop_device_link_server(self):
        self.dl_server.on_request = None
        return defer.maybeDeferred(self.dl_server_listener.stopListening)

    def _stop_client(self):
        results = []
        console_result = self._stop_console_client()
        if console_result is not None:
            results.append(console_result)
        dl_result = self._stop_device_link_client()
        if dl_result is not None:
            results.append(dl_result)
        return results

    def _stop_console_client(self):
        self.console_client_factory.receiver = None
        if self.console_client_connector is not None:
            self.console_client_connector.disconnect()
            return self.console_client_factory.on_connection_lost
        else:
            return None

    def _stop_device_link_client(self):
        if self.dl_client_connector is not None:
            self.dl_client.receiver = None
            return defer.maybeDeferred(self.dl_client_connector.stopListening)
        else:
            return None

    def _make_timeout(self, callback):
        from twisted.internet import reactor
        return reactor.callLater(self.timeout_value, callback, None)

    def _get_unexpecting_line_receiver(self, d):

        def got_line(line):
            timeout.cancel()
            from twisted.trial.unittest import FailTest
            d.errback(FailTest("Unexpected data:\n\t{0}.".format(line)))

        timeout = self._make_timeout(d.callback)
        return got_line

    def _get_expecting_line_receiver(self, expected_lines, d):

        def got_line(line):
            try:
                self.assertEqual(line, expected_lines.pop(0))
            except Exception as e:
                timeout.cancel()
                d.errback(e)
            else:
                if expected_lines:
                    return
                timeout.cancel()
                d.callback(None)

        def on_timeout(_):
            from twisted.trial.unittest import FailTest
            d.errback(FailTest(
                'Timed out, remaining lines:\n\t'+'\n\t'.join(expected_lines)))

        timeout = self._make_timeout(on_timeout)
        return got_line

    def _set_console_expecting_receiver(self, expected_lines, d):
        self.console_client.got_line = \
            self._get_expecting_line_receiver(expected_lines, d)

    def _set_dl_expecting_receiver(self, expected_lines, d):
        self.dl_client.receiver = \
            self._get_expecting_line_receiver(expected_lines, d)

    def _set_event_log_expecting_receiver(self, expected_lines, d):
        self.log_watcher.receiver = \
            self._get_expecting_line_receiver(expected_lines, d)

    @property
    def console_client_host_for_client(self):
        if self.console_server_listener is None:
            return (None, None)
        endpoint = self.console_server_listener.getHost()
        return endpoint.host, endpoint.port

    @property
    def device_link_host_for_client(self):
        if self.dl_server_listener is None:
            return (None, None)
        endpoint = self.dl_server_listener.getHost()
        return endpoint.host, endpoint.port


class BaseMiddlewareTestCase(BaseTestCase):

    console_client_factory_class = ConsoleClientFactory
    dl_client_class = DeviceLinkClient



# -*- coding: utf-8 -*-

import tempfile

from twisted.internet.defer import Deferred

from il2ds_middleware.ds_emulator.tests.base import BaseEmulatorTestCase


class EventLogTestCase(BaseEmulatorTestCase):

    def setUp(self):
        self.timeout_value = 0.5
        self.log_path = tempfile.mktemp()
        return super(EventLogTestCase, self).setUp()

    def test_event_log(self):
        pilots = self.service.getServiceNamed('pilots')
        missions = self.service.getServiceNamed('missions')
        static = self.service.getServiceNamed('static')

        responses = [
            "Mission: net/dogfight/test.mis is Playing\n",
            "Mission BEGIN\n",
            "user0 has connected\n",
            "user0:A6M2-21(0) seat occupied by user0 at 0 0\n",
            "user0:A6M2-21 loaded weapons '1xdt' fuel 100%\n",
            "user0:A6M2-21(0) was killed at 0 0\n",
            "0_Static destroyed by landscape at 0 0\n",
            "Mission END\n", ]
        d = Deferred()
        self._set_event_log_expecting_receiver(responses, d)

        missions.load("net/dogfight/test.mis")
        missions.begin()
        self.log_watcher.startService()

        static.spawn("0_Static")
        pilots.join("user0", "192.168.1.2")
        pilots.spawn("user0")
        pilots.kill("user0")
        static.destroy("0_Static")
        missions.end()
        return d



# -*- coding: utf-8 -*-

from twisted.application.service import Service
from twisted.internet import defer

from il2ds_middleware import service


class PilotService(service.PilotBaseService):

    def __init__(self):
        self.buffer = []

    def append_info(self, info):
        self.buffer.append(info)

    append_info = user_join = user_left = user_chat= \
    seat_occupied = weapons_loaded = was_killed = \
    was_shot_down = selected_army = went_to_menu = was_destroyed = \
    in_flight = landed = damaged = damaged_on_ground = \
    turned_wingtip_smokes = crashed = bailed_out = was_captured = \
    was_captured = was_wounded = was_heavily_wounded = removed = append_info


class ObjectsService(service.ObjectsBaseService):

    def __init__(self):
        self.buffer = []

    def was_destroyed(self, info):
        self.buffer.append(info)


class MissionService(service.MissionBaseService):

    def __init__(self):
        self.buffer = []

    def on_status_info(self, info):
        self.buffer.append(info)


class FakeLogWatchingService(Service):

    def stopService(self):
        Service.stopService(self)
        return defer.succeed(None)



# -*- coding: utf-8 -*-

import optparse

from twisted.application.internet import TimerService
from twisted.application.service import Service
from twisted.internet import defer, reactor

from zope.interface import implementer

from il2ds_middleware.interface.service import IMissionService
from il2ds_middleware.parser import ConsoleParser
from il2ds_middleware.protocol import ConsoleClientFactory
from il2ds_middleware.service import PilotBaseService, MissionBaseService


class PilotService(PilotBaseService):

    def __init__(self, missions):
        self.missions = missions

    def user_chat(self, (callsign, msg)):
        if msg == "<timeleft":
            self.client.chat_user(self.missions.time_left_str(), callsign)


class MissionService(MissionBaseService):

    client = None

    def __init__(self, mission, duration):
        self.mission = mission
        self.duration = duration
        self.time_checker = TimerService(1, self.check_time)
        self.time_left = 0
        self.time_to_notification = 0

    def startService(self):
        Service.startService(self)
        return self.reload_mission()

    def reload_mission(self):

        def on_destroyed(_):
            self.client.chat_all(
                "Loading mission \"{:}\".".format(self.mission))
            return self.client.mission_load(self.mission).addCallback(
                on_loaded)

        def on_loaded(_):
            return self.client.mission_begin().addCallback(self.on_playing)

        return self.client.mission_destroy().addCallback(on_destroyed)

    def on_playing(self, *args):
        self.client.chat_all(
            "Mission \"{:}\" is playing.".format(self.mission))
        self.time_left = self.duration
        self.time_to_notification == 0
        self.time_checker.startService()

    def check_time(self):
        if self.time_left == 0:
            self.time_checker.stopService()
            self.client.chat_all("Mission is ended.")
            return self.reload_mission()
        if self.time_to_notification == 0:
            self.client.chat_all(self.time_left_str())
            if not self.check_notification_time(15*60):
                if not self.check_notification_time(5*60):
                    if not self.check_notification_time(60):
                        if not self.check_notification_time(15):
                            if not self.check_notification_time(10, 5):
                                self.time_to_notification = 1
        self.time_left -= 1
        self.time_to_notification -= 1

    def time_left_str(self):
        m, s = divmod(self.time_left, 60)
        h, m = divmod(m, 60)
        return "Mission time left: %d:%02d:%02d" % (h, m, s)

    def check_notification_time(self, time_range, new_value=None):
        if self.time_left <= time_range:
            return False
        new_value = new_value or time_range
        value = self.time_left % new_value
        if value == 0:
            value = new_value
        self.time_to_notification = value
        return True


def parse_args():
    usage = """usage: %prog [--host=HOST] [--port=CSPORT]
    [--duration=DURATION] --mission=MISSION"""
    parser = optparse.OptionParser(usage)

    help = "The host to connect to. Default is localhost."
    parser.add_option('--host', help=help, default='localhost')

    help = "The console port to connect to. Default is 20000."
    parser.add_option('--port', type='int', default=20000, help=help)

    help = "Mission duration in seconds. Default is 3600."
    parser.add_option('--duration', type='int', default=3600, help=help)

    help = "Misssion to play (e.g., net/dogfight/test.mis)."
    parser.add_option('--mission', help=help)

    options, args = parser.parse_args()
    if not options.mission:
        parser.error("Misssion to play is not set.")
    return options


def main():
    options = parse_args()
    print "Working with server console on %s:%d." % (
        options.host, options.port)
    print "Rotating mission \"{:}\" with duration of {:} seconds.".format(
        options.mission, options.duration)

    def on_connected(client):
        missions.client = client
        pilots.client = client
        missions.startService()
        pilots.startService()

    def on_fail(err):
        print "Failed to connect: %s" % err.value
        reactor.stop()

    def on_connection_lost(err):
        print "Connection was lost."

    missions = MissionService(options.mission, options.duration)
    pilots = PilotService(missions)

    p = ConsoleParser((pilots, missions))
    f = ConsoleClientFactory(parser=p, timeout_value=1)
    f.on_connecting.addCallbacks(on_connected, on_fail)
    f.on_connection_lost.addErrback(on_connection_lost)
    reactor.connectTCP(options.host, options.port, f)
    reactor.run()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from twisted.internet import defer

from il2ds_middleware.ds_emulator.tests.base import BaseEmulatorTestCase


def expected_join_responses(channel, callsign, ip, port):
    return [
        "socket channel '{0}' start creating: ip {1}:{2}\\n".format(
            channel, ip, port),
        "Chat: --- {0} joins the game.\\n".format(callsign),
        "socket channel '{0}', ip {1}:{2}, {3}, "
        "is complete created.\\n".format(
            channel, ip, port, callsign), ]


def expected_leave_responses(channel, callsign, ip, port):
    return [
        "socketConnection with {0}:{1} on channel {2} lost.  "
        "Reason: \\n".format(
            ip, port, channel),
        "Chat: --- {0} has left the game.\\n".format(
            callsign), ]


def expected_kick_responses(channel, callsign, ip, port):
    return [
        "socketConnection with {0}:{1} on channel {2} lost.  "
        "Reason: You have been kicked from the server.\\n".format(
            ip, port, channel),
        "Chat: --- {0} has left the game.\\n".format(
            callsign), ]


def expected_load_responses(path):
    return [
        "Loading mission {0}...\\n".format(path),
        "Load bridges\\n",
        "Load static objects\\n",
        "##### House without collision (3do/Tree/Tree2.sim)\\n",
        "##### House without collision (3do/Buildings/Port/Floor/live.sim)\\n",
        "##### House without collision (3do/Buildings/Port/BaseSegment/"
            "live.sim)\\n",
        "Mission: {0} is Loaded\\n".format(path), ]


class CommonsTestCase(BaseEmulatorTestCase):

    def test_connect(self):
        self.assertNotEqual(self.service.client, None)

    def test_disconnect(self):
        self.console_server_factory.on_connection_lost.addBoth(
            lambda _: self.assertEqual(self.service.client, None))
        self.console_client_connector.disconnect()

    def test_receive_line(self):
        d = defer.Deferred()
        responses = ["test\\n", ]
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        self.console_server.message("test")
        return d

    def test_unknown_command(self):
        d = defer.Deferred()
        responses = ["Command not found: abracadabracadabr\\n", ]
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        self.console_client.sendLine("abracadabracadabr")
        return d

    def test_server_info(self):

        def do_test():
            responses =[
                "Type: Local server\\n",
                "Name: Server\\n",
                "Description: \\n", ]
            d = defer.Deferred()
            self.console_client_factory.receiver = \
                self._get_expecting_line_receiver(responses, d)
            d.addCallback(change_info)
            self.console_client.sendLine("server")
            return d

        def change_info(_):
            self.service.set_server_info(
                "Test server", "This is a server emulator")
            responses =[
                "Type: Local server\\n",
                "Name: Test server\\n",
                "Description: This is a server emulator\\n", ]
            d = defer.Deferred()
            self.console_client_factory.receiver = \
                self._get_expecting_line_receiver(responses, d)
            self.console_client.sendLine("server")
            return d

        return do_test()


class PilotsTestCase(BaseEmulatorTestCase):

    def setUp(self):
        r = super(PilotsTestCase, self).setUp()
        self.srvc = self.service.getServiceNamed('pilots')
        return r

    def tearDown(self):
        self.srvc = None
        return super(PilotsTestCase, self).tearDown()

    def _get_pilots_count_checker(self, expected_count):
        def check(_):
            self.assertEqual(len(self.srvc.pilots), expected_count)
        return check

    def test_join(self):
        responses = expected_join_responses(
            1, "user0", "192.168.1.2", self.srvc.port)
        responses.extend(expected_join_responses(
            3, "user1", "192.168.1.3", self.srvc.port))

        d = defer.Deferred()
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        d.addCallback(self._get_pilots_count_checker(2))

        self.srvc.join("user0", "192.168.1.2")
        self.srvc.join("user1", "192.168.1.3")
        return d

    def test_leave(self):
        responses = expected_join_responses(
            1, "user0", "192.168.1.2", self.srvc.port)
        responses.extend(expected_join_responses(
            3, "user1", "192.168.1.3", self.srvc.port))
        responses.extend(expected_leave_responses(
            1, "user0", "192.168.1.2", self.srvc.port))

        d = defer.Deferred()
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        d.addCallback(self._get_pilots_count_checker(1))
        self.srvc.join("user0", "192.168.1.2")
        self.srvc.join("user1", "192.168.1.3")
        self.srvc.leave("user0")
        self.srvc.leave("fake_user")
        return d

    def test_kick(self):
        responses = expected_join_responses(
            1, "user0", "192.168.1.2", self.srvc.port)
        responses.extend(expected_join_responses(
            3, "user1", "192.168.1.3", self.srvc.port))
        responses.extend(expected_kick_responses(
            1, "user0", "192.168.1.2", self.srvc.port))

        d = defer.Deferred()
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        d.addCallback(self._get_pilots_count_checker(1))
        self.srvc.join("user0", "192.168.1.2")
        self.srvc.join("user1", "192.168.1.3")
        self.console_client.sendLine("kick user0")
        return d


class MissionsTestCase(BaseEmulatorTestCase):

    def setUp(self):
        r = super(MissionsTestCase, self).setUp()
        self.srvc = self.service.getServiceNamed('missions')
        return r

    def tearDown(self):
        self.srvc = None
        return super(MissionsTestCase, self).tearDown()

    def test_no_mission(self):
        d = defer.Deferred()
        responses = ["Mission NOT loaded\\n", ]
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        self.console_client.sendLine("mission")
        return d

    def test_load_mission(self):
        responses = expected_load_responses("net/dogfight/test.mis")
        responses.append(responses[-1])

        d = defer.Deferred()
        self._set_console_expecting_receiver(responses, d)
        self.console_client.sendLine("mission LOAD net/dogfight/test.mis")
        self.console_client.sendLine("mission")
        return d

    def test_begin_mission(self):
        responses = ["ERROR mission: Mission NOT loaded\\n", ]
        responses.extend(expected_load_responses("net/dogfight/test.mis"))
        responses.append("Mission: net/dogfight/test.mis is Playing\\n")
        responses.append(responses[-1])

        d = defer.Deferred()
        self._set_console_expecting_receiver(responses, d)
        self.console_client.sendLine("mission BEGIN")
        self.console_client.sendLine("mission LOAD net/dogfight/test.mis")
        self.console_client.sendLine("mission BEGIN")
        self.console_client.sendLine("mission")
        return d

    def test_end_mission(self):
        responses = ["ERROR mission: Mission NOT loaded\\n", ]
        responses.extend(expected_load_responses("net/dogfight/test.mis"))
        responses.append("Mission: net/dogfight/test.mis is Playing\\n")
        responses.append("Mission: net/dogfight/test.mis is Loaded\\n")

        d = defer.Deferred()
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        self.console_client.sendLine("mission END")
        self.console_client.sendLine("mission LOAD net/dogfight/test.mis")
        self.console_client.sendLine("mission BEGIN")
        self.console_client.sendLine("mission END")
        self.console_client.sendLine("mission END END END")
        self.console_client.sendLine("mission")
        return d

    def test_destroy_mission(self):
        responses = ["ERROR mission: Mission NOT loaded\\n", ]
        responses.extend(expected_load_responses("net/dogfight/test.mis"))
        responses.extend(expected_load_responses("net/dogfight/test.mis"))
        responses.append("Mission: net/dogfight/test.mis is Playing\\n")
        responses.append("Mission NOT loaded\\n")

        d = defer.Deferred()
        self.console_client_factory.receiver = \
            self._get_expecting_line_receiver(responses, d)
        self.console_client.sendLine("mission DESTROY")
        self.console_client.sendLine("mission LOAD net/dogfight/test.mis")
        self.console_client.sendLine("mission DESTROY")
        self.console_client.sendLine("mission LOAD net/dogfight/test.mis")
        self.console_client.sendLine("mission BEGIN")
        self.console_client.sendLine("mission DESTROY")
        self.console_client.sendLine("mission")
        return d



# -*- coding: utf-8 -*-

from twisted.internet import defer
from twisted.trial.unittest import TestCase

from il2ds_middleware.tests.base import BaseTestCase
from il2ds_middleware.ds_emulator.service import RootService
from il2ds_middleware.ds_emulator.protocol import (DeviceLinkServerProtocol,
    ConsoleServerFactory, )
from il2ds_middleware.ds_emulator.tests.protocol import (ConsoleClientFactory,
    ConsoleClient, DeviceLinkClient, )
from il2ds_middleware.ds_emulator.tests.service import LogWatchingService


class BaseEmulatorTestCase(BaseTestCase):

    console_client_factory_class = ConsoleClientFactory
    dl_client_class = DeviceLinkClient
    log_watcher_class = LogWatchingService



# -*- coding: utf-8 -*-

from twisted.application.service import Service
from twisted.trial.unittest import TestCase

from il2ds_middleware.constants import MISSION_STATUS
from il2ds_middleware.service import MissionService
from il2ds_middleware.tests.service import FakeLogWatchingService


class MissionServiceTestCase(TestCase):

    def setUp(self):
        self.log_watcher = FakeLogWatchingService()
        self.srvc = MissionService(self.log_watcher)
        self.srvc.startService()

    def tearDown(self):
        return self.srvc.stopService()

    def test_mission_begun(self):
        self.srvc.on_status_info((MISSION_STATUS.PLAYING, "test.mis"))
        self.assertTrue(self.log_watcher.running)
        self.assertEqual(self.srvc.mission, "test.mis")
        self.assertEqual(self.srvc.status, MISSION_STATUS.PLAYING)

    def test_mission_ended(self):
        self.srvc.mission = "test.mis"
        self.srvc.status = MISSION_STATUS.PLAYING
        self.log_watcher.startService()
        self.srvc.on_status_info((MISSION_STATUS.LOADED, "test.mis"))
        self.assertFalse(self.log_watcher.running)
        self.assertEqual(self.srvc.mission, "test.mis")
        self.assertEqual(self.srvc.status, MISSION_STATUS.LOADED)



# -*- coding: utf-8 -*-

import optparse

from twisted.application.internet import TimerService
from twisted.application.service import MultiService
from twisted.internet import defer, reactor

from il2ds_middleware.parser import (ConsoleParser, DeviceLinkParser,
    EventLogParser, )
from il2ds_middleware.protocol import ConsoleClientFactory, DeviceLinkClient
from il2ds_middleware import service


class PilotService(service.PilotBaseService):

    dlink = None

    def user_left(self, info):
        self.dlink.refresh_radar()

    def seat_occupied(self, info):
        self.dlink.refresh_radar()

    def was_killed(self, info):
        self.dlink.refresh_radar()

    def was_shot_down(self, info):
        self.dlink.refresh_radar()

    def went_to_menu(self, info):
        self.dlink.refresh_radar()

    def user_chat(self, info):
        print "%s says: %s" % info


class ObjectsService(service.ObjectsBaseService):
    pass


class MissionService(service.MissionService):

    dlink = None

    def began(self, info=None):
        service.MissionService.began(self, info)
        self.dlink.refresh_radar()

    def ended(self, info=None):
        service.MissionService.ended(self, info)
        self.dlink.refresh_radar()


class PilotRadarService(TimerService):

    dlink = None

    def __init__(self, interval=10):
        TimerService.__init__(self, interval, self.do_watch)

    def do_watch(self):
        self.dlink.all_pilots_pos().addCallbacks(
            self.on_response, self.on_error)

    def on_response(self, response):
        print "%s Got %s coordinates." % ("="*3, len(response))
        if not response:
            return
        print "{:^15} | {:^10} | {:^10} | {:^10}".format(
            "callsign", "x", "y", "z")
        for data in response:
            print "{:<15} | {:<10} | {:<10} | {:<10}".format(
                data['callsign'],
                data['pos']['x'], data['pos']['y'], data['pos']['z'], )

    def on_error(err):
        print "Getting pilots positions failed: %s." % err.value
        reactor.stop()


def parse_args():
    usage = """usage: %prog [--host=HOST] [--csport=CSPORT] [--dlport=DLPORT]
    [--frequency=FREQUENCY] --log=LOG"""
    parser = optparse.OptionParser(usage)

    help = "The host to connect to. Default is localhost."
    parser.add_option('--host', help=help, default='localhost')

    help = "The console port to connect to. Default is 20000."
    parser.add_option('--csport', type='int', default=20000, help=help)

    help = "The Device Link port to connect to. Default is 10000."
    parser.add_option('--dlport', type='int', default=10000, help=help)

    help = "Radar refreshing frequency. Default is 5 seconds."
    parser.add_option('--frequency', type='int', default=5, help=help)

    help = "Path to events log file."
    parser.add_option('--log', help=help)

    options, args = parser.parse_args()
    if not options.log:
        parser.error("Path to events log is not set.")
    return options


def main():
    options = parse_args()
    (host, port) = (options.host, options.csport)
    dl_address = (options.host, options.dlport)

    print "Working with"
    print "Server console on %s:%d." % (host, port)
    print "Device Link on %s:%d." % dl_address

    root = MultiService()
    pilots = PilotService()
    pilots.setServiceParent(root)

    objects = ObjectsService()
    objects.setServiceParent(root)

    radar = PilotRadarService(options.frequency)
    radar.setServiceParent(root)

    parser = EventLogParser((pilots, objects))
    log_watcher = service.LogWatchingService(options.log, parser=parser)
    missions = MissionService(log_watcher)
    missions.setServiceParent(root)

    def on_start(_):
        dl_client.refresh_radar()
        pilots.dlink = dl_client
        radar.dlink = dl_client
        missions.dlink = dl_client
        root.startService()
        root.client.mission_status()

    def on_connected(client):
        root.client = client
        pilots.client = client
        objects.client = client
        d = dl_client.on_start.addCallback(on_start)
        reactor.listenUDP(0, dl_client)
        return d

    def on_fail(err):
        print "Failed to connect: %s" % err.value
        reactor.stop()

    def on_connection_lost(err):
        print "Connection was lost."

    parser = ConsoleParser((pilots, missions))
    f = ConsoleClientFactory(parser)
    dl_client = DeviceLinkClient(dl_address, DeviceLinkParser())
    f.on_connecting.addCallbacks(on_connected, on_fail)
    f.on_connection_lost.addErrback(on_connection_lost)

    reactor.connectTCP(host, port, f)
    reactor.run()


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name='il2ds-middleware',
    version='0.8.1',
    description='High-level access to IL-2 FB Dedicated Server.',
    license='MIT',
    url='https://github.com/IL2HorusTeam/il2ds-middleware',
    author='Alexander Oblovatniy',
    author_email='oblovatniy@gmail.com',
    packages=find_packages(exclude=["examples", ]),
    install_requires=[i.strip() for i in open("requirements.pip").readlines()],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries',
    ],
)



# -*- coding: utf-8 -*-

import re

from twisted.python import log

from zope.interface import implementer

from il2ds_middleware.constants import MISSION_STATUS, PILOT_LEAVE_REASON
from il2ds_middleware.interface.parser import (IDeviceLinkParser,
    IEventLogParser, IConsoleParser, )
from il2ds_middleware.regex import *


@implementer(IConsoleParser)
class ConsolePassthroughParser(object):

    def passthrough(self, data):
        return data

    parse_line = server_info = mission_status = passthrough
    mission_load = mission_destroy = mission_status
    mission_begin = mission_end = mission_status


@implementer(IConsoleParser)
class ConsoleParser(object):

    _buffer = None

    def __init__(self, (pilot_service, mission_service)):
        self.pilot_service = pilot_service
        self.mission_service = mission_service

    def parse_line(self, line):
        while True:
            if self.user_chat(line):
                break
            if self.user_joined(line):
                break
            if self.user_left(line):
                break
            if self._mission_status(line):
                break
            return False
        return True

    def server_info(self, lines):
        result = {}
        for line in lines:
            key, value = line.split(':')
            result[key.strip().lower()] = value.strip()
        return result

    def mission_status(self, lines):
        for line in lines:
            info = self._mission_status(line)
            if info:
                return info
        return lines

    def _mission_status(self, line):
        info = None
        if line == "Mission NOT loaded":
            info = (MISSION_STATUS.NOT_LOADED, None, )
        elif line.endswith("is Loaded"):
            info = (MISSION_STATUS.LOADED, line.split()[1], )
        elif line.endswith("is Playing"):
            info = (MISSION_STATUS.PLAYING, line.split()[1], )
        if info:
            self.mission_service.on_status_info(info)
        return info

    def user_joined(self, line):
        m = re.match(RX_USER_JOIN, line)
        if not m:
            return False
        else:
            groups = m.groups()
            info = {
                'channel': int(groups[0]),
                'ip': groups[1],
                'callsign': groups[2],
            }
            self.pilot_service.user_join(info)
            return True

    def user_left(self, line):
        m = re.match(RX_USER_LEFT, line)
        if m:
            groups = m.groups()
            reason = (PILOT_LEAVE_REASON.KICKED if 'kicked' in groups[2]
                else PILOT_LEAVE_REASON.DISCONNECTED)
            self._buffer = {
                'ip': groups[0],
                'channel': int(groups[1]),
                'reason': reason,
            }
            return True
        if line.endswith("has left the game.") and self._buffer:
            info, self._buffer = self._buffer, None
            info['callsign'] = line.split()[2]
            self.pilot_service.user_left(info)
            return True
        return False

    def user_chat(self, line):
        m = re.match(RX_USER_CHAT, line)
        if not m:
            return False
        else:
            callsign, msg = m.groups()
            if callsign != "Server":
                info = (callsign, msg.decode('unicode-escape'))
                self.pilot_service.user_chat(info)
            return True


@implementer(IEventLogParser)
class EventLogPassthroughParser(object):

    def passthrough(self, data):
        return data

    parse_line = seat_occupied = weapons_loaded = was_killed = \
    was_shot_down = selected_army = went_to_menu = was_destroyed = \
    in_flight = landed = damaged = damaged_on_ground = \
    turned_wingtip_smokes = crashed = bailed_out = was_captured = \
    was_captured = was_wounded = was_heavily_wounded = removed = passthrough


@implementer(IEventLogParser)
class EventLogParser(object):

    def __init__(self, (pilot_service, objects_service)):
        ps = pilot_service
        obs = objects_service
        self.parsers = (
            (RX_SEAT_OCCUPIED, self.seat_occupied, ps.seat_occupied),
            (RX_SELECTED_ARMY, self.selected_army, ps.selected_army),
            (RX_DESTROYED, self.was_destroyed, obs.was_destroyed),
            (RX_WEAPONS_LOADED, self.weapons_loaded, ps.weapons_loaded),
            (RX_KILLED, self.was_killed, ps.was_killed),
            (RX_SHOT_DOWN, self.was_shot_down, ps.was_shot_down),
            (RX_WENT_TO_MENU, self.went_to_menu, ps.went_to_menu),
            (RX_IN_FLIGHT, self.in_flight, ps.in_flight),
            (RX_LANDED, self.landed, ps.landed),
            (RX_CRASHED, self.crashed, ps.crashed),
            (RX_DAMAGED, self.damaged, ps.damaged),
            (RX_DAMAGED_ON_GROUND, self.damaged_on_ground,
                ps.damaged_on_ground),
            (RX_TURNED_WINGTIP_SMOKES, self.turned_wingtip_smokes,
                ps.turned_wingtip_smokes),
            (RX_BAILED_OUT, self.bailed_out, ps.bailed_out),
            (RX_WAS_CAPTURED, self.was_captured, ps.was_captured),
            (RX_WAS_WOUNDED, self.was_wounded, ps.was_wounded),
            (RX_WAS_HEAVILY_WOUNDED, self.was_heavily_wounded,
                ps.was_heavily_wounded),
            (RX_REMOVED, self.removed, ps.removed),
        )

    def parse_line(self, line):
        for rx, parser, dst in self.parsers:
            m = re.match(rx, line)
            if m:
                dst(parser(m.groups()))
                return True
        return False

    def _seat_event(self, groups):
        return {
            'callsign': groups[0],
            'aircraft': groups[1],
            'seat': int(groups[2]),
            'pos': self._pos((groups[3], groups[4])),
        }

    def _aircraft_event(self, groups):
        return {
            'callsign': groups[0],
            'aircraft': groups[1],
            'pos': self._pos((groups[2], groups[3])),
        }

    def _actor(self, data):
        info = data.split(':')
        is_user = len(info) == 2
        actor = {
            'is_user': is_user,
        }
        if is_user:
            actor['callsign'] = info[0]
            actor['aircraft'] = info[1]
        else:
            actor['name'] = info[0]
        return actor

    def _pos(self, (x, y)):
        return {
            'x': float(x),
            'y': float(y),
        }

    seat_occupied = was_killed = bailed_out = was_captured = was_wounded = \
    was_heavily_wounded = _seat_event

    in_flight = damaged_on_ground = crashed = removed = _aircraft_event

    def weapons_loaded(self, groups):
        return {
            'callsign': groups[0],
            'aircraft': groups[1],
            'weapons': groups[2],
            'fuel': int(groups[3]),
        }

    def was_shot_down(self, groups):
        return {
            'victim': {
                'callsign': groups[0],
                'aircraft': groups[1],
            },
            'attacker': self._actor(groups[2]),
            'pos': self._pos((groups[3], groups[4])),
        }

    def selected_army(self, groups):
        return {
            'callsign': groups[0],
            'army': groups[1],
            'pos': self._pos((groups[2], groups[3])),
        }

    def went_to_menu(self, groups):
        return {
            'callsign': groups[0],
        }

    def was_destroyed(self, groups):
        return {
            'victim': groups[0],
            'attacker': groups[1],
            'pos': self._pos((groups[2], groups[3])),
        }

    def landed(self, groups):
        info = self._actor(groups[0])
        info.update({
            'pos': self._pos((groups[1], groups[2])),
        })
        return info

    def damaged(self, groups):
        return {
            'victim': {
                'callsign': groups[0],
                'aircraft': groups[1],
            },
            'attacker': self._actor(groups[2]),
            'pos': self._pos((groups[3], groups[4])),
        }

    def turned_wingtip_smokes(self, groups):
        return {
            'callsign': groups[0],
            'aircraft': groups[1],
            'state': groups[2],
            'pos': self._pos((groups[3], groups[4])),
        }


@implementer(IDeviceLinkParser)
class DeviceLinkPassthroughParser(object):

    def passthrough(self, data):
        return data

    pilot_count = pilot_pos = all_pilots_pos = passthrough
    static_count = static_pos = all_static_pos = passthrough


@implementer(IDeviceLinkParser)
class DeviceLinkParser(object):

    def pilot_count(self, data):
        return int(data)

    def pilot_pos(self, data):
        return self._parse_pos(data, 'callsign', strip_idx=True)

    def all_pilots_pos(self, datas):
        return map(self.pilot_pos, datas)

    static_count = pilot_count

    def static_pos(self, data):
        return self._parse_pos(data)

    def all_static_pos(self, datas):
        return map(self.static_pos, datas)

    def _parse_pos(self, data, name_attr='name', strip_idx=False):
        idx, info = data.split(':')
        attr, x, y, z = info.split(';')
        if strip_idx:
            attr = attr[:attr.rindex("_")]
        return {
            'idx': int(idx),
            name_attr: attr,
            'pos': {
                'x': int(x),
                'y': int(y),
                'z': int(z),
            },
        }
